import json
from typing import Iterator, Type, TypeVar

import requests
from dacite import Config, from_dict
from qwak.exceptions.qwak_decode_exception import QwakDecodeException
from qwak.exceptions.qwak_external_exception import QwakExternalException
from qwak.llmops.generation.streaming import Stream

_SSEvent = TypeVar("_SSEvent")


class BaseSSEDecoder(Stream[_SSEvent]):
    _response: requests.Response
    _iterator: Iterator[_SSEvent]

    def __init__(self, response: requests.Response, parse_to: Type[_SSEvent]):
        if response.encoding is None:
            response.encoding = "utf-8"

        self._parse_to = parse_to

        self._response = response
        self._iterator = self.__decode(response=response)

    def _is_chunk_empty(self, chunk: str) -> bool:
        return not chunk or chunk == "" or chunk.startswith(":") or chunk == ":"

    def _raise_on_chunk_error(self, chunk: str):
        is_error: bool = False
        try:
            maybe_error = json.loads(chunk)
            if "error" in maybe_error:
                is_error = True
        except Exception:  # nosec
            pass

        if chunk.strip().startswith("error"):
            is_error = True

        if is_error:
            raise QwakExternalException(message=chunk)

    def __decode(self, response: requests.Response) -> Iterator[_SSEvent]:
        chunk: str
        for chunk in response.iter_lines(chunk_size=None, decode_unicode=True):
            if self._is_chunk_empty(chunk=chunk):
                continue

            self._raise_on_chunk_error(chunk=chunk)

            try:
                if chunk.strip().startswith("[DONE]"):
                    break

                _, _, content = chunk.partition(":")

                if content.strip().startswith("[DONE]"):
                    break

                yield self._parse_event(event=content)
            except Exception as e:
                raise QwakDecodeException(f"Failed to decode event: {chunk}") from e

    def __iter__(self) -> Iterator[_SSEvent]:
        for chunk in self._iterator:
            yield chunk

    def __next__(self) -> _SSEvent:
        return self._iterator.__next__()

    def _parse_event(self, event: str) -> _SSEvent:
        return from_dict(
            data_class=self._parse_to,
            data=json.loads(event),
            config=Config(check_types=False),
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from _qwak_proto.qwak.model_group import model_group_pb2 as qwak_dot_model__group_dot_model__group__pb2


class ModelGroupManagementServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateIfNotExistsModelGroup = channel.unary_unary(
                '/qwak.model_groups.management.ModelGroupManagementService/CreateIfNotExistsModelGroup',
                request_serializer=qwak_dot_model__group_dot_model__group__pb2.CreateIfNotExistsModelGroupRequest.SerializeToString,
                response_deserializer=qwak_dot_model__group_dot_model__group__pb2.ModelGroupBriefInfoResponse.FromString,
                )
        self.CreateRepositoriesForModelGroup = channel.unary_unary(
                '/qwak.model_groups.management.ModelGroupManagementService/CreateRepositoriesForModelGroup',
                request_serializer=qwak_dot_model__group_dot_model__group__pb2.CreateRepositoriesForModelGroupRequest.SerializeToString,
                response_deserializer=qwak_dot_model__group_dot_model__group__pb2.CreateRepositoriesForModelGroupResponse.FromString,
                )
        self.GetModelGroupInfo = channel.unary_unary(
                '/qwak.model_groups.management.ModelGroupManagementService/GetModelGroupInfo',
                request_serializer=qwak_dot_model__group_dot_model__group__pb2.GetModelGroupInfoRequest.SerializeToString,
                response_deserializer=qwak_dot_model__group_dot_model__group__pb2.ModelGroupBriefInfoResponse.FromString,
                )
        self.GetModelGroupSystemPullImageSecret = channel.unary_unary(
                '/qwak.model_groups.management.ModelGroupManagementService/GetModelGroupSystemPullImageSecret',
                request_serializer=qwak_dot_model__group_dot_model__group__pb2.GetModelGroupSystemPullImageSecretRequest.SerializeToString,
                response_deserializer=qwak_dot_model__group_dot_model__group__pb2.SystemPullSecretInfoNamesResponse.FromString,
                )


class ModelGroupManagementServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateIfNotExistsModelGroup(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateRepositoriesForModelGroup(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetModelGroupInfo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetModelGroupSystemPullImageSecret(self, request, context):
        """*
        This end point gets a list of model group names and returns a list of SystemPullSecretInfo
        that contains model group name and the secret pull image with the "system-" addition to the pull secret name
        For example:
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ModelGroupManagementServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateIfNotExistsModelGroup': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateIfNotExistsModelGroup,
                    request_deserializer=qwak_dot_model__group_dot_model__group__pb2.CreateIfNotExistsModelGroupRequest.FromString,
                    response_serializer=qwak_dot_model__group_dot_model__group__pb2.ModelGroupBriefInfoResponse.SerializeToString,
            ),
            'CreateRepositoriesForModelGroup': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateRepositoriesForModelGroup,
                    request_deserializer=qwak_dot_model__group_dot_model__group__pb2.CreateRepositoriesForModelGroupRequest.FromString,
                    response_serializer=qwak_dot_model__group_dot_model__group__pb2.CreateRepositoriesForModelGroupResponse.SerializeToString,
            ),
            'GetModelGroupInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetModelGroupInfo,
                    request_deserializer=qwak_dot_model__group_dot_model__group__pb2.GetModelGroupInfoRequest.FromString,
                    response_serializer=qwak_dot_model__group_dot_model__group__pb2.ModelGroupBriefInfoResponse.SerializeToString,
            ),
            'GetModelGroupSystemPullImageSecret': grpc.unary_unary_rpc_method_handler(
                    servicer.GetModelGroupSystemPullImageSecret,
                    request_deserializer=qwak_dot_model__group_dot_model__group__pb2.GetModelGroupSystemPullImageSecretRequest.FromString,
                    response_serializer=qwak_dot_model__group_dot_model__group__pb2.SystemPullSecretInfoNamesResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'qwak.model_groups.management.ModelGroupManagementService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ModelGroupManagementService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateIfNotExistsModelGroup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.model_groups.management.ModelGroupManagementService/CreateIfNotExistsModelGroup',
            qwak_dot_model__group_dot_model__group__pb2.CreateIfNotExistsModelGroupRequest.SerializeToString,
            qwak_dot_model__group_dot_model__group__pb2.ModelGroupBriefInfoResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateRepositoriesForModelGroup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.model_groups.management.ModelGroupManagementService/CreateRepositoriesForModelGroup',
            qwak_dot_model__group_dot_model__group__pb2.CreateRepositoriesForModelGroupRequest.SerializeToString,
            qwak_dot_model__group_dot_model__group__pb2.CreateRepositoriesForModelGroupResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetModelGroupInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.model_groups.management.ModelGroupManagementService/GetModelGroupInfo',
            qwak_dot_model__group_dot_model__group__pb2.GetModelGroupInfoRequest.SerializeToString,
            qwak_dot_model__group_dot_model__group__pb2.ModelGroupBriefInfoResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetModelGroupSystemPullImageSecret(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.model_groups.management.ModelGroupManagementService/GetModelGroupSystemPullImageSecret',
            qwak_dot_model__group_dot_model__group__pb2.GetModelGroupSystemPullImageSecretRequest.SerializeToString,
            qwak_dot_model__group_dot_model__group__pb2.SystemPullSecretInfoNamesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from abc import ABC, abstractmethod
from dataclasses import dataclass
from datetime import datetime, timezone
from typing import List, Optional, Set, Union

from _qwak_proto.qwak.feature_store.features.execution_pb2 import (
    BackfillExecutionSpec as ProtoBackfillExecutionSpec,
)
from _qwak_proto.qwak.feature_store.features.feature_set_types_pb2 import (
    BackfillBatchDataSourceSpec as ProtoBackfillBatchDataSourceSpec,
    BackfillDataSourceSpec as ProtoBackfillDataSourceSpec,
    BackfillSpec as ProtoBackfillSpec,
)
from _qwak_proto.qwak.feature_store.sources.batch_pb2 import (
    BatchSource as ProtoBatchSource,
)
from google.protobuf.timestamp_pb2 import Timestamp as ProtoTimestamp
from qwak.clients.feature_store import FeatureRegistryClient
from qwak.exceptions import QwakException
from qwak.feature_store._common.artifact_utils import ArtifactSpec, ArtifactsUploader
from qwak.feature_store._common.feature_set_utils import get_batch_source_for_featureset
from qwak.feature_store.feature_sets.execution_spec import ClusterTemplate
from qwak.feature_store.feature_sets.transformations import SparkSqlTransformation

_BACKFILL_ = "_qwak_backfill_specification"


@dataclass
class DataSourceBackfillSpec(ABC):
    data_source_name: str

    @abstractmethod
    def _to_proto(self, feature_registry: FeatureRegistryClient):
        pass

    @classmethod
    def _from_proto(cls, proto: ProtoBackfillDataSourceSpec):
        function_mapping = {"batch_data_source_spec": BackfillBatchDataSourceSpec}

        backfill_source_type: str = proto.WhichOneof("type")

        if backfill_source_type in function_mapping:
            function_class = function_mapping.get(backfill_source_type)
            return function_class._from_proto(proto)

        raise QwakException(
            f"Got unsupported backfill source type {backfill_source_type} for streaming backfill"
        )


@dataclass
class BackfillBatchDataSourceSpec(DataSourceBackfillSpec):
    start_datetime: Optional[datetime] = None
    end_datetime: Optional[datetime] = None

    def _to_proto(
        self, feature_registry: FeatureRegistryClient
    ) -> ProtoBackfillBatchDataSourceSpec:
        start_timestamp: Optional[ProtoTimestamp] = None
        end_timestamp: Optional[ProtoTimestamp] = None

        if self.end_datetime:
            end_timestamp = ProtoTimestamp()
            end_timestamp.FromDatetime(self.end_datetime.astimezone(timezone.utc))

        if self.start_datetime:
            start_timestamp = ProtoTimestamp()
            start_timestamp.FromDatetime(self.start_datetime.astimezone(timezone.utc))

        proto_data_source: ProtoBatchSource = get_batch_source_for_featureset(
            batch_ds_name=self.data_source_name, feature_registry=feature_registry
        )

        return ProtoBackfillBatchDataSourceSpec(
            data_source=proto_data_source,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
        )

    @classmethod
    def _from_proto(
        cls, proto: ProtoBackfillDataSourceSpec
    ) -> "BackfillBatchDataSourceSpec":
        start_datetime: Optional[datetime] = None
        end_datetime: Optional[datetime] = None

        batch_backfill_spec: ProtoBackfillBatchDataSourceSpec = (
            proto.batch_data_source_spec
        )

        proto_start_timestamp: ProtoTimestamp = batch_backfill_spec.start_timestamp
        proto_end_timestamp: ProtoTimestamp = batch_backfill_spec.end_timestamp

        start_datetime = datetime.fromtimestamp(
            proto_start_timestamp.seconds + proto_start_timestamp.nanos / 1e9
        )

        end_datetime = datetime.fromtimestamp(
            proto_end_timestamp.seconds + proto_end_timestamp.nanos / 1e9
        )

        return cls(
            data_source_name=batch_backfill_spec.data_source.name,
            start_datetime=start_datetime,
            end_datetime=end_datetime,
        )


@dataclass
class StreamingBackfill:
    start_datetime: datetime
    end_datetime: datetime
    data_sources_specs: List[DataSourceBackfillSpec]
    transform: "SparkSqlTransformation"
    cluster_template: Optional[ClusterTemplate] = ClusterTemplate.SMALL

    def __post_init__(self):
        if not self.data_sources_specs:
            raise QwakException(
                "Trying to create a streaming backfill with no data sources. "
                "At least one data source has to be provided when trying to create a streaming backfill."
            )

        if not self.start_datetime or not self.end_datetime:
            raise QwakException(
                "For backfill, start_datetime and end_datetime are mandatory fields."
            )

        if type(self.transform) is not SparkSqlTransformation:
            raise QwakException(
                "For backfill, only Spark SQL transformation type is currently supported"
            )

        self._validate_unique_sources()

    def _validate_unique_sources(self):
        source_names: List[str] = [
            data_source.data_source_name for data_source in self.data_sources_specs
        ]
        duplicates: Set[str] = {
            item for item in source_names if source_names.count(item) > 1
        }
        if duplicates:
            raise QwakException(
                f"A specific data source can only appear once per backfill definition. "
                f"Found these duplicates: {', '.join(set(duplicates))}"
            )

    def _validate_tile_size(self, initial_tile_size: int):
        if self.end_datetime.timestamp() % initial_tile_size != 0:
            raise QwakException(
                f"Chosen backfill end datetime is invalid,"
                f" it has to be exactly dividable by slice size of {initial_tile_size} seconds."
            )

    def _to_proto(
        self,
        feature_registry: FeatureRegistryClient,
        featureset_name: str,
        original_instance_module_path: str,
    ) -> ProtoBackfillSpec:
        artifact_url: Optional[str] = None
        artifact_spec: Optional[ArtifactSpec] = ArtifactsUploader.get_artifact_spec(
            transformation=self.transform,
            featureset_name=f"{featureset_name}-backfill",
            __instance_module_path__=original_instance_module_path,
        )

        if artifact_spec:
            artifact_url = ArtifactsUploader.upload(artifact_spec)

        end_timestamp = ProtoTimestamp()
        end_timestamp.FromDatetime(self.end_datetime.astimezone(timezone.utc))

        start_timestamp = ProtoTimestamp()
        start_timestamp.FromDatetime(self.start_datetime.astimezone(timezone.utc))

        return ProtoBackfillSpec(
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            execution_spec=ProtoBackfillExecutionSpec(
                **{"cluster_template": ClusterTemplate.to_proto(self.cluster_template)}
            ),
            transformation=self.transform._to_proto(artifact_path=artifact_url),
            data_source_specs=[
                ProtoBackfillDataSourceSpec(
                    batch_data_source_spec=data_source_spec._to_proto(
                        feature_registry=feature_registry
                    )
                )
                for data_source_spec in self.data_sources_specs
            ],
        )

    @classmethod
    def _from_proto(cls, proto: ProtoBackfillSpec):
        datetime.fromtimestamp(
            proto.start_timestamp.seconds + proto.start_timestamp.nanos / 1e9
        )

        data_sources_specs = [
            BackfillBatchDataSourceSpec._from_proto(ds)
            for ds in proto.data_source_specs
        ]

        return cls(
            start_datetime=datetime.fromtimestamp(
                proto.start_timestamp.seconds + proto.start_timestamp.nanos / 1e9
            ),
            end_datetime=datetime.fromtimestamp(
                proto.end_timestamp.seconds + proto.end_timestamp.nanos / 1e9
            ),
            data_sources_specs=data_sources_specs,
            transform=SparkSqlTransformation._from_proto(
                proto.transformation.sql_transformation
            ),
        )

    @staticmethod
    def _get_normalized_backfill_sources_spec(
        data_sources: Union[List[str], List[DataSourceBackfillSpec]],
    ) -> List[DataSourceBackfillSpec]:
        # reformat all data source specs to 'DataSourceBackfillSpec'
        return [
            (
                BackfillBatchDataSourceSpec(data_source_name=data_source)
                if isinstance(data_source, str)
                else data_source
            )
            for data_source in data_sources
        ]

    @classmethod
    def set_streaming_backfill_on_function(
        cls,
        function,
        start_date: datetime,
        end_date: datetime,
        data_sources: Union[List[str], List[DataSourceBackfillSpec]],
        backfill_transformation: SparkSqlTransformation,
        backfill_cluster_template: Optional[ClusterTemplate] = ClusterTemplate.SMALL,
    ):
        setattr(
            function,
            _BACKFILL_,
            cls(
                start_datetime=start_date,
                end_datetime=end_date,
                data_sources_specs=StreamingBackfill._get_normalized_backfill_sources_spec(
                    data_sources
                ),
                transform=backfill_transformation,
                cluster_template=backfill_cluster_template,
            ),
        )

    @staticmethod
    def get_streaming_backfill_from_function(function):
        return getattr(function, _BACKFILL_, None)



from typing import Optional

import grpc
from _qwak_proto.qwak.data_versioning.data_versioning_pb2 import (
    DataTagFilter,
    DataTagSpec,
)
from _qwak_proto.qwak.data_versioning.data_versioning_service_pb2 import (
    GetModelDataTagsRequest,
    RegisterDataTagRequest,
)
from _qwak_proto.qwak.data_versioning.data_versioning_service_pb2_grpc import (
    DataVersioningManagementServiceStub,
)
from dependency_injector.wiring import Provide
from qwak.exceptions import QwakException
from qwak.inner.di_configuration import QwakContainer


class DataVersioningManagementClient:
    def __init__(self, grpc_channel=Provide[QwakContainer.core_grpc_channel]):
        self._data_management_service = DataVersioningManagementServiceStub(
            grpc_channel
        )

    def register_data_tag(
        self,
        model_id: str,
        tag: str,
        build_id: Optional[str] = "",
        extension: Optional[str] = "",
    ) -> None:
        """
        Register data tag to service
        Args:
            build_id: build id to save tag by.
            model_id: model id to save tag by.
            tag: tag to save.
            extension: The file extension

        Returns:

        """
        try:
            self._data_management_service.RegisterDataTag(
                RegisterDataTagRequest(
                    data_tag_spec=(
                        DataTagSpec(
                            build_id=build_id,
                            model_id=model_id,
                            tag=tag,
                            extension_type=extension,
                        )
                    )
                )
            )
        except grpc.RpcError as e:
            if e.args[0].code != grpc.StatusCode.ALREADY_EXISTS:
                raise QwakException(
                    f"Failed to register data tag, error is {e.details()}"
                )

    def get_model_data_tags(
        self, model_id: str, build_id: str, data_tag_filter: DataTagFilter = None
    ):
        try:
            return self._data_management_service.GetModelDataTags(
                GetModelDataTagsRequest(
                    model_id=model_id, build_id=build_id, filter=data_tag_filter
                )
            )
        except grpc.RpcError as e:
            raise QwakException(
                f"Failed to list model data tags, error is {e.details()}"
            )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/automation/v1/common.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fqwak/automation/v1/common.proto\x12\x12qwak.automation.v1\"G\n\x06Metric\x12\x33\n\nsql_metric\x18\x01 \x01(\x0b\x32\x1d.qwak.automation.v1.SqlMetricH\x00\x42\x08\n\x06metric\"\x1e\n\tSqlMetric\x12\x11\n\tsql_query\x18\x01 \x01(\t*\x84\x01\n\x18MetricThresholdDirection\x12\x1c\n\x18INVALID_METRIC_DIRECTION\x10\x00\x12\t\n\x05\x42\x45LOW\x10\x01\x12\t\n\x05\x41\x42OVE\x10\x02\x12\n\n\x06\x45QUALS\x10\x03\x12\x13\n\x0f\x42\x45LOW_OR_EQUALS\x10\x04\x12\x13\n\x0f\x41\x42OVE_OR_EQUALS\x10\x05\x42!\n\x1d\x63om.qwak.ai.automation.api.v1P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.automation.v1.common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\035com.qwak.ai.automation.api.v1P\001'
  _globals['_METRICTHRESHOLDDIRECTION']._serialized_start=161
  _globals['_METRICTHRESHOLDDIRECTION']._serialized_end=293
  _globals['_METRIC']._serialized_start=55
  _globals['_METRIC']._serialized_end=126
  _globals['_SQLMETRIC']._serialized_start=128
  _globals['_SQLMETRIC']._serialized_end=158
# @@protoc_insertion_point(module_scope)



from ._input_checks_utility import (
    is_not_none,
    is_valid_thread_number,
    user_input_validation,
    validate_not_folder_paths,
    validate_path_exists,
)
from ._storage_utils import calculate_sha2, calc_content_sha2
from ._url_utils import (
    assemble_artifact_url,
    join_url,
)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _JobState:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _JobStateEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_JobState.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID: _JobState.ValueType  # 0
    """Job state"""
    PENDING: _JobState.ValueType  # 1
    """Job is pending"""
    RUNNING: _JobState.ValueType  # 2
    """Job is running"""
    COMPLETED: _JobState.ValueType  # 3
    """Job has completed"""
    FAILED: _JobState.ValueType  # 4
    """Job has failed"""
    TOO_EARLY: _JobState.ValueType  # 5
    """Job is too early"""

class JobState(_JobState, metaclass=_JobStateEnumTypeWrapper): ...

INVALID: JobState.ValueType  # 0
"""Job state"""
PENDING: JobState.ValueType  # 1
"""Job is pending"""
RUNNING: JobState.ValueType  # 2
"""Job is running"""
COMPLETED: JobState.ValueType  # 3
"""Job has completed"""
FAILED: JobState.ValueType  # 4
"""Job has failed"""
TOO_EARLY: JobState.ValueType  # 5
"""Job is too early"""
global___JobState = JobState

class JobRecord(google.protobuf.message.Message):
    """Feature Store Job Record.
    Represents a job generated from featureset
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOB_ID_FIELD_NUMBER: builtins.int
    RUN_ID_FIELD_NUMBER: builtins.int
    FEATURESET_NAME_FIELD_NUMBER: builtins.int
    FEATURESET_ID_FIELD_NUMBER: builtins.int
    ENVIRONMENT_ID_FIELD_NUMBER: builtins.int
    TRY_NUMBER_FIELD_NUMBER: builtins.int
    JOB_STATE_FIELD_NUMBER: builtins.int
    CREATION_TIME_FIELD_NUMBER: builtins.int
    START_TIME_FIELD_NUMBER: builtins.int
    END_TIME_FIELD_NUMBER: builtins.int
    INGESTION_WINDOW_FIELD_NUMBER: builtins.int
    EXECUTION_ID_FIELD_NUMBER: builtins.int
    INGESTION_MESSAGE_FIELD_NUMBER: builtins.int
    job_id: builtins.str
    """Unique id of the job"""
    run_id: builtins.int
    """Run id for the job"""
    featureset_name: builtins.str
    """Featureset name"""
    featureset_id: builtins.str
    """Featureset id"""
    environment_id: builtins.str
    """Environment_id"""
    try_number: builtins.int
    """Try count"""
    job_state: global___JobState.ValueType
    """State of the job - whether it's running, failed, pending, etc"""
    @property
    def creation_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Creation time"""
    @property
    def start_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Start time"""
    @property
    def end_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """End time"""
    @property
    def ingestion_window(self) -> global___IngestionWindow:
        """Ingestion window information"""
    execution_id: builtins.int
    """The execution id of the record"""
    ingestion_message: builtins.str
    """Ingestion message"""
    def __init__(
        self,
        *,
        job_id: builtins.str = ...,
        run_id: builtins.int = ...,
        featureset_name: builtins.str = ...,
        featureset_id: builtins.str = ...,
        environment_id: builtins.str = ...,
        try_number: builtins.int = ...,
        job_state: global___JobState.ValueType = ...,
        creation_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        start_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        end_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        ingestion_window: global___IngestionWindow | None = ...,
        execution_id: builtins.int = ...,
        ingestion_message: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["creation_time", b"creation_time", "end_time", b"end_time", "ingestion_window", b"ingestion_window", "start_time", b"start_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["creation_time", b"creation_time", "end_time", b"end_time", "environment_id", b"environment_id", "execution_id", b"execution_id", "featureset_id", b"featureset_id", "featureset_name", b"featureset_name", "ingestion_message", b"ingestion_message", "ingestion_window", b"ingestion_window", "job_id", b"job_id", "job_state", b"job_state", "run_id", b"run_id", "start_time", b"start_time", "try_number", b"try_number"]) -> None: ...

global___JobRecord = JobRecord

class IngestionWindow(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INGESTED_DATA_START_TIME_FIELD_NUMBER: builtins.int
    INGESTED_DATA_END_TIME_FIELD_NUMBER: builtins.int
    INGESTED_DATA_UPDATED_ROWS_FIELD_NUMBER: builtins.int
    SCHEDULED_EXECUTION_TIME_FIELD_NUMBER: builtins.int
    @property
    def ingested_data_start_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Ingested data window start time"""
    @property
    def ingested_data_end_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Ingested data window end time"""
    ingested_data_updated_rows: builtins.int
    """Ingested data number of updated rows"""
    @property
    def scheduled_execution_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Ingested data scheduled execution date"""
    def __init__(
        self,
        *,
        ingested_data_start_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        ingested_data_end_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        ingested_data_updated_rows: builtins.int = ...,
        scheduled_execution_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["ingested_data_end_time", b"ingested_data_end_time", "ingested_data_start_time", b"ingested_data_start_time", "scheduled_execution_time", b"scheduled_execution_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ingested_data_end_time", b"ingested_data_end_time", "ingested_data_start_time", b"ingested_data_start_time", "ingested_data_updated_rows", b"ingested_data_updated_rows", "scheduled_execution_time", b"scheduled_execution_time"]) -> None: ...

global___IngestionWindow = IngestionWindow

class FeaturesetLevelJobsSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_ID_FIELD_NUMBER: builtins.int
    TOTAL_NUM_AFFECTED_ROWS_FIELD_NUMBER: builtins.int
    featureset_id: builtins.str
    """Featureset Id"""
    total_num_affected_rows: builtins.int
    """Number of affected rows"""
    def __init__(
        self,
        *,
        featureset_id: builtins.str = ...,
        total_num_affected_rows: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_id", b"featureset_id", "total_num_affected_rows", b"total_num_affected_rows"]) -> None: ...

global___FeaturesetLevelJobsSummary = FeaturesetLevelJobsSummary



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/build_settings/build_settings.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(qwak/build_settings/build_settings.proto\x12\x17\x63om.qwak.build_settings\"c\n\rBuildSettings\x12R\n\x18model_repository_setting\x18\x01 \x01(\x0b\x32\x30.com.qwak.build_settings.ModelRepositorySettings\"\xcc\x02\n\x17ModelRepositorySettings\x12!\n\x19\x63ontainer_registry_prefix\x18\x01 \x01(\t\x12Q\n\x17repository_settings_ecr\x18\x02 \x01(\x0b\x32..com.qwak.build_settings.RepositorySettingsECRH\x00\x12U\n\x19repository_settings_jfrog\x18\x03 \x01(\x0b\x32\x30.com.qwak.build_settings.RepositorySettingsJFrogH\x00\x12Q\n\x17repository_settings_gcr\x18\x04 \x01(\x0b\x32..com.qwak.build_settings.RepositorySettingsGCRH\x00\x42\x11\n\x0frepository_type\"\xe2\x01\n\x17RepositorySettingsJFrog\x12P\n\x17\x61uthentication_settings\x18\x01 \x01(\x0b\x32/.com.qwak.build_settings.AuthenticationSettings\x12\x1a\n\x12jfrog_project_name\x18\x02 \x01(\t\x12Y\n\x1c\x65xternal_repository_settings\x18\x03 \x01(\x0b\x32\x33.com.qwak.build_settings.ExternalRepositorySettings\"\xb3\x01\n\x1a\x45xternalRepositorySettings\x12\x19\n\x11python_repository\x18\x01 \x03(\t\x12\x1e\n\x16huggingface_repository\x18\x02 \x01(\t\x12*\n\"allow_external_python_repositories\x18\x03 \x01(\x08\x12.\n&virtual_python_dependencies_repository\x18\x04 \x01(\t\"9\n\x16\x41uthenticationSettings\x12\r\n\x05token\x18\x01 \x01(\t\x12\x10\n\x08username\x18\x02 \x01(\t\"\x17\n\x15RepositorySettingsECR\"\x17\n\x15RepositorySettingsGCRB/\n\x17\x63om.qwak.build_settingsB\x12\x42uildSettingsProtoP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.build_settings.build_settings_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027com.qwak.build_settingsB\022BuildSettingsProtoP\001'
  _globals['_BUILDSETTINGS']._serialized_start=69
  _globals['_BUILDSETTINGS']._serialized_end=168
  _globals['_MODELREPOSITORYSETTINGS']._serialized_start=171
  _globals['_MODELREPOSITORYSETTINGS']._serialized_end=503
  _globals['_REPOSITORYSETTINGSJFROG']._serialized_start=506
  _globals['_REPOSITORYSETTINGSJFROG']._serialized_end=732
  _globals['_EXTERNALREPOSITORYSETTINGS']._serialized_start=735
  _globals['_EXTERNALREPOSITORYSETTINGS']._serialized_end=914
  _globals['_AUTHENTICATIONSETTINGS']._serialized_start=916
  _globals['_AUTHENTICATIONSETTINGS']._serialized_end=973
  _globals['_REPOSITORYSETTINGSECR']._serialized_start=975
  _globals['_REPOSITORYSETTINGSECR']._serialized_end=998
  _globals['_REPOSITORYSETTINGSGCR']._serialized_start=1000
  _globals['_REPOSITORYSETTINGSGCR']._serialized_end=1023
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/feature_store/sources/batch.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&qwak/feature_store/sources/batch.proto\x12\x1aqwak.feature.store.sources\x1a\x1fgoogle/protobuf/timestamp.proto\"\xa5\x07\n\x0b\x42\x61tchSource\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x1b\n\x13\x64\x61te_created_column\x18\x03 \x01(\t\x12@\n\x0c\x61thenaSource\x18\x04 \x01(\x0b\x32(.qwak.feature.store.sources.AthenaSourceH\x00\x12>\n\x0bmongoSource\x18\x05 \x01(\x0b\x32\'.qwak.feature.store.sources.MongoSourceH\x00\x12:\n\tcsvSource\x18\x06 \x01(\x0b\x32%.qwak.feature.store.sources.CsvSourceH\x00\x12\x46\n\x0fsnowflakeSource\x18\x07 \x01(\x0b\x32+.qwak.feature.store.sources.SnowflakeSourceH\x00\x12\x42\n\rparquetSource\x18\x08 \x01(\x0b\x32).qwak.feature.store.sources.ParquetSourceH\x00\x12<\n\njdbcSource\x18\t \x01(\x0b\x32&.qwak.feature.store.sources.JdbcSourceH\x00\x12\x42\n\rverticaSource\x18\n \x01(\x0b\x32).qwak.feature.store.sources.VerticaSourceH\x00\x12\x44\n\x0e\x62igquerySource\x18\x0b \x01(\x0b\x32*.qwak.feature.store.sources.BigquerySourceH\x00\x12N\n\x13\x65lasticsearchSource\x18\x0c \x01(\x0b\x32/.qwak.feature.store.sources.ElasticsearchSourceH\x00\x12H\n\x10\x63lickhouseSource\x18\r \x01(\x0b\x32,.qwak.feature.store.sources.ClickhouseSourceH\x00\x12V\n\x17\x66\x65\x61turesetOfflineSource\x18\x0e \x01(\x0b\x32\x33.qwak.feature.store.sources.FeaturesetOfflineSourceH\x00\x12J\n\x11tableFormatSource\x18\x0f \x01(\x0b\x32-.qwak.feature.store.sources.TableFormatSourceH\x00\x42\x06\n\x04type\"\xae\x04\n\x17\x46ileSystemConfiguration\x12X\n\x14\x61ws_s3_configuration\x18\x01 \x01(\x0b\x32\x38.qwak.feature.store.sources.AwsS3FileSystemConfigurationH\x00\x12P\n\x10\x61ws_s3_anonymous\x18\x02 \x01(\x0b\x32\x34.qwak.feature.store.sources.AnonymousS3ConfigurationH\x00\x12W\n aws_s3_assume_role_configuration\x18\x03 \x01(\x0b\x32+.qwak.feature.store.sources.AwsS3AssumeRoleH\x00\x12M\n\x13gcs_unauthenticated\x18\x04 \x01(\x0b\x32..qwak.feature.store.sources.GcsUnauthenticatedH\x00\x12N\n\x14gcs_user_credentials\x18\x05 \x01(\x0b\x32..qwak.feature.store.sources.GcsUserCredentialsH\x00\x12g\n!gcs_service_account_impersonation\x18\x06 \x01(\x0b\x32:.qwak.feature.store.sources.GcsServiceAccountImpersonationH\x00\x42\x06\n\x04type\"#\n\x0f\x41wsS3AssumeRole\x12\x10\n\x08role_arn\x18\x01 \x01(\t\"\x91\x01\n\x1c\x41wsS3FileSystemConfiguration\x12\x1e\n\x16\x61\x63\x63\x65ss_key_secret_name\x18\x01 \x01(\t\x12\x1e\n\x16secret_key_secret_name\x18\x02 \x01(\t\x12!\n\x19session_token_secret_name\x18\x03 \x01(\t\x12\x0e\n\x06\x62ucket\x18\x04 \x01(\t\"\x1a\n\x18\x41nonymousS3Configuration\"\x92\x03\n\nJdbcSource\x12\x0b\n\x03url\x18\x03 \x01(\t\x12\x1c\n\x14username_secret_name\x18\x01 \x01(\t\x12\x1c\n\x14password_secret_name\x18\x02 \x01(\t\x12\x10\n\x08\x64\x62_table\x18\x04 \x01(\t\x12\r\n\x05query\x18\x05 \x01(\t\x12>\n\x0bmysqlSource\x18\x06 \x01(\x0b\x32\'.qwak.feature.store.sources.MysqlSourceH\x00\x12H\n\x10postgresqlSource\x18\x07 \x01(\x0b\x32,.qwak.feature.store.sources.PostgresqlSourceH\x00\x12\x44\n\x0eredshiftSource\x18\x08 \x01(\x0b\x32*.qwak.feature.store.sources.RedshiftSourceH\x00\x12\x42\n\x0c\x61thenaSource\x18\t \x01(\x0b\x32*.qwak.feature.store.sources.AthenaSourceV1H\x00\x42\x06\n\x04type\"\r\n\x0bMysqlSource\"\x12\n\x10PostgresqlSource\"\x89\x01\n\x10\x43lickhouseSource\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x1c\n\x14username_secret_name\x18\x02 \x01(\t\x12\x1c\n\x14password_secret_name\x18\x03 \x01(\t\x12\x0f\n\x05table\x18\x04 \x01(\tH\x00\x12\r\n\x03sql\x18\x05 \x01(\tH\x00\x42\x0c\n\nquery_type\"i\n\x0eRedshiftSource\x12\x0f\n\x07\x64\x62_user\x18\x01 \x01(\t\x12\x14\n\x0ciam_role_arn\x18\x02 \x01(\t\x12\x15\n\raccess_key_id\x18\x03 \x01(\t\x12\x19\n\x11secret_access_key\x18\x04 \x01(\t\"\x8d\x03\n\x0e\x41thenaSourceV1\x12\x12\n\naws_region\x18\x01 \x01(\t\x12I\n\x12\x61ws_authentication\x18\x02 \x01(\x0b\x32-.qwak.feature.store.sources.AwsAuthentication\x12\x1a\n\x12s3_output_location\x18\x03 \x01(\t\x12R\n\x16\x64\x61te_partition_columns\x18\x04 \x01(\x0b\x32\x30.qwak.feature.store.sources.DatePartitionColumnsH\x00\x12g\n!time_fragmented_partition_columns\x18\x05 \x01(\x0b\x32:.qwak.feature.store.sources.TimeFragmentedPartitionColumnsH\x00\x12\x13\n\tworkgroup\x18\x06 \x01(\tH\x01\x42\x18\n\x16time_partition_columnsB\x14\n\x12optional_workgroup\"E\n\x14\x44\x61tePartitionColumns\x12\x18\n\x10\x64\x61te_column_name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x61te_format\x18\x02 \x01(\t\"\xb5\x02\n\x1eTimeFragmentedPartitionColumns\x12M\n\x15year_partition_column\x18\x01 \x01(\x0b\x32..qwak.feature.store.sources.YearFragmentColumn\x12Q\n\x16month_partition_column\x18\x02 \x01(\x0b\x32/.qwak.feature.store.sources.MonthFragmentColumnH\x00\x12M\n\x14\x64\x61y_partition_column\x18\x03 \x01(\x0b\x32-.qwak.feature.store.sources.DayFragmentColumnH\x01\x42\x11\n\x0fmonth_partitionB\x0f\n\rday_partition\"\xa4\x01\n\x12YearFragmentColumn\x12\x13\n\x0b\x63olumn_name\x18\x01 \x01(\t\x12`\n\x1dnumeric_column_representation\x18\x02 \x01(\x0b\x32\x37.qwak.feature.store.sources.NumericColumnRepresentationH\x00\x42\x17\n\x15\x63olumn_representation\"\x87\x02\n\x13MonthFragmentColumn\x12\x13\n\x0b\x63olumn_name\x18\x01 \x01(\t\x12`\n\x1dnumeric_column_representation\x18\x02 \x01(\x0b\x32\x37.qwak.feature.store.sources.NumericColumnRepresentationH\x00\x12`\n\x1dtextual_column_representation\x18\x03 \x01(\x0b\x32\x37.qwak.feature.store.sources.TextualColumnRepresentationH\x00\x42\x17\n\x15\x63olumn_representation\"\xa3\x01\n\x11\x44\x61yFragmentColumn\x12\x13\n\x0b\x63olumn_name\x18\x01 \x01(\t\x12`\n\x1dnumeric_column_representation\x18\x02 \x01(\x0b\x32\x37.qwak.feature.store.sources.NumericColumnRepresentationH\x00\x42\x17\n\x15\x63olumn_representation\"\x1d\n\x1bNumericColumnRepresentation\"\x1d\n\x1bTextualColumnRepresentation\"/\n\x0c\x41thenaSource\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\x12\r\n\x05table\x18\x02 \x01(\t\"\xa0\x02\n\x0e\x42igquerySource\x12\x1f\n\x17\x63redentials_secret_name\x18\x01 \x01(\t\x12\x0f\n\x07\x64\x61taset\x18\x02 \x01(\t\x12\r\n\x05table\x18\x03 \x01(\t\x12\x0f\n\x07project\x18\x04 \x01(\t\x12\x16\n\x0eparent_project\x18\x05 \x01(\t\x12\x0b\n\x03sql\x18\x06 \x01(\t\x12\x15\n\rviews_enabled\x18\x07 \x01(\x08\x12#\n\x17materialization_dataset\x18\x08 \x01(\tB\x02\x18\x01\x12#\n\x17materialization_project\x18\t \x01(\tB\x02\x18\x01\x12\x36\n*materialization_expiration_time_in_minutes\x18\n \x01(\tB\x02\x18\x01\"\xa3\x01\n\tCsvSource\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x17\n\x0fquote_character\x18\x02 \x01(\t\x12\x18\n\x10\x65scape_character\x18\x03 \x01(\t\x12U\n\x18\x66ilesystem_configuration\x18\x04 \x01(\x0b\x32\x33.qwak.feature.store.sources.FileSystemConfiguration\"\xab\x01\n\x0bMongoSource\x12\r\n\x05hosts\x18\x01 \x01(\t\x12\x1c\n\x14username_secret_name\x18\x02 \x01(\t\x12\x1c\n\x14password_secret_name\x18\x03 \x01(\t\x12\x10\n\x08\x64\x61tabase\x18\x04 \x01(\t\x12\x12\n\ncollection\x18\x05 \x01(\t\x12\x19\n\x11\x63onnection_params\x18\x06 \x01(\t\x12\x10\n\x08protocol\x18\x07 \x01(\t\"\xea\x01\n\x0fSnowflakeSource\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x1c\n\x14username_secret_name\x18\x02 \x01(\t\x12\"\n\x14password_secret_name\x18\x03 \x01(\tB\x02\x18\x01H\x00\x12%\n\x1bpem_private_key_secret_name\x18\t \x01(\tH\x00\x12\x10\n\x08\x64\x61tabase\x18\x04 \x01(\t\x12\x0e\n\x06schema\x18\x05 \x01(\t\x12\x11\n\twarehouse\x18\x06 \x01(\t\x12\r\n\x05table\x18\x07 \x01(\t\x12\r\n\x05query\x18\x08 \x01(\tB\r\n\x0b\x61uth_secret\"t\n\rParquetSource\x12\x0c\n\x04path\x18\x01 \x01(\t\x12U\n\x18\x66ilesystem_configuration\x18\x02 \x01(\x0b\x32\x33.qwak.feature.store.sources.FileSystemConfiguration\"\x98\x01\n\rVerticaSource\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\x12\x10\n\x08\x64\x61tabase\x18\x03 \x01(\t\x12\x0e\n\x06schema\x18\x04 \x01(\t\x12\r\n\x05table\x18\x05 \x01(\t\x12\x1c\n\x14username_secret_name\x18\x06 \x01(\t\x12\x1c\n\x14password_secret_name\x18\x07 \x01(\t\"\xbc\x01\n\x13\x45lasticsearchSource\x12\r\n\x05nodes\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\x12\x10\n\x08resource\x18\x03 \x01(\t\x12\r\n\x05query\x18\x04 \x01(\t\x12\x16\n\x0e\x65xclude_fields\x18\x05 \x01(\t\x12\x13\n\x0bparse_dates\x18\x06 \x01(\x08\x12\x1c\n\x14username_secret_name\x18\x07 \x01(\t\x12\x1c\n\x14password_secret_name\x18\x08 \x01(\t\"X\n\x11TableFormatSource\x12\x34\n\x07\x63\x61talog\x18\x01 \x01(\x0b\x32#.qwak.feature.store.sources.Catalog\x12\r\n\x05query\x18\x02 \x01(\t\"S\n\x07\x43\x61talog\x12@\n\x0cunityCatalog\x18\x01 \x01(\x0b\x32(.qwak.feature.store.sources.UnityCatalogH\x00\x42\x06\n\x04type\"g\n\x0cUnityCatalog\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0f\n\x07\x63\x61talog\x18\x02 \x01(\t\x12+\n!personal_access_token_secret_name\x18\x03 \x01(\tH\x00\x42\x0b\n\tauth_type\"(\n\x17\x46\x65\x61turesetOfflineSource\x12\r\n\x05\x61lias\x18\x01 \x01(\t\"\xe9\x01\n\x11\x41wsAuthentication\x12\x61\n\x1e\x61ws_assume_role_authentication\x18\x01 \x01(\x0b\x32\x37.qwak.feature.store.sources.AwsAssumeRoleAuthenticationH\x00\x12i\n%aws_static_credentials_authentication\x18\x02 \x01(\x0b\x32\x38.qwak.feature.store.sources.AwsCredentialsAuthenticationH\x00\x42\x06\n\x04type\"/\n\x1b\x41wsAssumeRoleAuthentication\x12\x10\n\x08role_arn\x18\x01 \x01(\t\"^\n\x1c\x41wsCredentialsAuthentication\x12\x1e\n\x16\x61\x63\x63\x65ss_key_secret_name\x18\x01 \x01(\t\x12\x1e\n\x16secret_key_secret_name\x18\x02 \x01(\t\"\x14\n\x12GcsUnauthenticated\"y\n\x12GcsUserCredentials\x12\x1d\n\x15\x63lient_id_secret_name\x18\x01 \x01(\t\x12!\n\x19\x63lient_secret_secret_name\x18\x02 \x01(\t\x12!\n\x19refresh_token_secret_name\x18\x03 \x01(\t\">\n\x1eGcsServiceAccountImpersonation\x12\x1c\n\x14service_account_user\x18\x01 \x01(\tBX\n%com.qwak.ai.feature.store.sources.apiP\x01Z-qwak/featurestore/sources;featurestoresourcesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.feature_store.sources.batch_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n%com.qwak.ai.feature.store.sources.apiP\001Z-qwak/featurestore/sources;featurestoresources'
  _BIGQUERYSOURCE.fields_by_name['materialization_dataset']._options = None
  _BIGQUERYSOURCE.fields_by_name['materialization_dataset']._serialized_options = b'\030\001'
  _BIGQUERYSOURCE.fields_by_name['materialization_project']._options = None
  _BIGQUERYSOURCE.fields_by_name['materialization_project']._serialized_options = b'\030\001'
  _BIGQUERYSOURCE.fields_by_name['materialization_expiration_time_in_minutes']._options = None
  _BIGQUERYSOURCE.fields_by_name['materialization_expiration_time_in_minutes']._serialized_options = b'\030\001'
  _SNOWFLAKESOURCE.fields_by_name['password_secret_name']._options = None
  _SNOWFLAKESOURCE.fields_by_name['password_secret_name']._serialized_options = b'\030\001'
  _globals['_BATCHSOURCE']._serialized_start=104
  _globals['_BATCHSOURCE']._serialized_end=1037
  _globals['_FILESYSTEMCONFIGURATION']._serialized_start=1040
  _globals['_FILESYSTEMCONFIGURATION']._serialized_end=1598
  _globals['_AWSS3ASSUMEROLE']._serialized_start=1600
  _globals['_AWSS3ASSUMEROLE']._serialized_end=1635
  _globals['_AWSS3FILESYSTEMCONFIGURATION']._serialized_start=1638
  _globals['_AWSS3FILESYSTEMCONFIGURATION']._serialized_end=1783
  _globals['_ANONYMOUSS3CONFIGURATION']._serialized_start=1785
  _globals['_ANONYMOUSS3CONFIGURATION']._serialized_end=1811
  _globals['_JDBCSOURCE']._serialized_start=1814
  _globals['_JDBCSOURCE']._serialized_end=2216
  _globals['_MYSQLSOURCE']._serialized_start=2218
  _globals['_MYSQLSOURCE']._serialized_end=2231
  _globals['_POSTGRESQLSOURCE']._serialized_start=2233
  _globals['_POSTGRESQLSOURCE']._serialized_end=2251
  _globals['_CLICKHOUSESOURCE']._serialized_start=2254
  _globals['_CLICKHOUSESOURCE']._serialized_end=2391
  _globals['_REDSHIFTSOURCE']._serialized_start=2393
  _globals['_REDSHIFTSOURCE']._serialized_end=2498
  _globals['_ATHENASOURCEV1']._serialized_start=2501
  _globals['_ATHENASOURCEV1']._serialized_end=2898
  _globals['_DATEPARTITIONCOLUMNS']._serialized_start=2900
  _globals['_DATEPARTITIONCOLUMNS']._serialized_end=2969
  _globals['_TIMEFRAGMENTEDPARTITIONCOLUMNS']._serialized_start=2972
  _globals['_TIMEFRAGMENTEDPARTITIONCOLUMNS']._serialized_end=3281
  _globals['_YEARFRAGMENTCOLUMN']._serialized_start=3284
  _globals['_YEARFRAGMENTCOLUMN']._serialized_end=3448
  _globals['_MONTHFRAGMENTCOLUMN']._serialized_start=3451
  _globals['_MONTHFRAGMENTCOLUMN']._serialized_end=3714
  _globals['_DAYFRAGMENTCOLUMN']._serialized_start=3717
  _globals['_DAYFRAGMENTCOLUMN']._serialized_end=3880
  _globals['_NUMERICCOLUMNREPRESENTATION']._serialized_start=3882
  _globals['_NUMERICCOLUMNREPRESENTATION']._serialized_end=3911
  _globals['_TEXTUALCOLUMNREPRESENTATION']._serialized_start=3913
  _globals['_TEXTUALCOLUMNREPRESENTATION']._serialized_end=3942
  _globals['_ATHENASOURCE']._serialized_start=3944
  _globals['_ATHENASOURCE']._serialized_end=3991
  _globals['_BIGQUERYSOURCE']._serialized_start=3994
  _globals['_BIGQUERYSOURCE']._serialized_end=4282
  _globals['_CSVSOURCE']._serialized_start=4285
  _globals['_CSVSOURCE']._serialized_end=4448
  _globals['_MONGOSOURCE']._serialized_start=4451
  _globals['_MONGOSOURCE']._serialized_end=4622
  _globals['_SNOWFLAKESOURCE']._serialized_start=4625
  _globals['_SNOWFLAKESOURCE']._serialized_end=4859
  _globals['_PARQUETSOURCE']._serialized_start=4861
  _globals['_PARQUETSOURCE']._serialized_end=4977
  _globals['_VERTICASOURCE']._serialized_start=4980
  _globals['_VERTICASOURCE']._serialized_end=5132
  _globals['_ELASTICSEARCHSOURCE']._serialized_start=5135
  _globals['_ELASTICSEARCHSOURCE']._serialized_end=5323
  _globals['_TABLEFORMATSOURCE']._serialized_start=5325
  _globals['_TABLEFORMATSOURCE']._serialized_end=5413
  _globals['_CATALOG']._serialized_start=5415
  _globals['_CATALOG']._serialized_end=5498
  _globals['_UNITYCATALOG']._serialized_start=5500
  _globals['_UNITYCATALOG']._serialized_end=5603
  _globals['_FEATURESETOFFLINESOURCE']._serialized_start=5605
  _globals['_FEATURESETOFFLINESOURCE']._serialized_end=5645
  _globals['_AWSAUTHENTICATION']._serialized_start=5648
  _globals['_AWSAUTHENTICATION']._serialized_end=5881
  _globals['_AWSASSUMEROLEAUTHENTICATION']._serialized_start=5883
  _globals['_AWSASSUMEROLEAUTHENTICATION']._serialized_end=5930
  _globals['_AWSCREDENTIALSAUTHENTICATION']._serialized_start=5932
  _globals['_AWSCREDENTIALSAUTHENTICATION']._serialized_end=6026
  _globals['_GCSUNAUTHENTICATED']._serialized_start=6028
  _globals['_GCSUNAUTHENTICATED']._serialized_end=6048
  _globals['_GCSUSERCREDENTIALS']._serialized_start=6050
  _globals['_GCSUSERCREDENTIALS']._serialized_end=6171
  _globals['_GCSSERVICEACCOUNTIMPERSONATION']._serialized_start=6173
  _globals['_GCSSERVICEACCOUNTIMPERSONATION']._serialized_end=6235
# @@protoc_insertion_point(module_scope)



import os

home_dir = os.path.expanduser("~")

JFROG_CLI_CONFIG_FILE_PATH = os.path.join(home_dir, ".jfrog", "jfrog-cli.conf.v6")
CONFIG_FILE_PATH = os.path.join(home_dir, ".frogml", "config.json")

FROG_ML_CONFIG_USER = "user"
FROG_ML_CONFIG_ARTIFACTORY_URL = "artifactory_url"
FROG_ML_CONFIG_PASSWORD = "password"  # nosec B105
FROG_ML_CONFIG_ACCESS_TOKEN = "access_token"  # nosec B105
FROG_ML_DEFAULT_HTTP_THREADS_COUNT = 5
FROG_ML_MAX_CHARS_FOR_NAME = 60

SERVER_ID = "server_id"

JFROG_CLI_CONFIG_ARTIFACTORY_URL = "artifactoryUrl"
JFROG_CLI_CONFIG_URL = "url"
JFROG_CLI_CONFIG_USER = "user"
JFROG_CLI_CONFIG_PASSWORD = "password"  # nosec B105
JFROG_CLI_CONFIG_ACCESS_TOKEN = "accessToken"  # nosec B105

MODEL = "model"
ROOT_FROGML_MODEL_UI_DIRECTORY = "models"
MODEL_UI_DIRECTORY = "model"
MODEL_METADATA_FILE_NAME = "model-manifest.json"
BODY_PART_MODEL_MANIFEST_STREAM = "modelManifest"

DATASET = "dataset"
ROOT_FROGML_DATASET_UI_DIRECTORY = "datasets"
DATASET_UI_DIRECTORY = "dataset"
DATASET_METADATA_FILE_NAME = "dataset-manifest.json"
BODY_PART_DATASET_MANIFEST_STREAM = "datasetManifest"

CHECKSUM_SHA2_HEADER = "X-Checksum-Sha256"

JFML_THREAD_COUNT = "JFML_THREAD_COUNT"
JF_URL = "JF_URL"
JF_ACCESS_TOKEN = "JF_ACCESS_TOKEN"  # nosec B105

FROG_ML_IGNORED_FILES = [
    ".DS_Store",
    "CVS",
    ".cvsignore",
    "SCCS",
    "vssver.scc",
    ".svn",
    ".git",
    ".gitignore",
    ".gitattributes",
    ".gitmodules",
    ".gitkeep",
    ".gitconfig",
    MODEL_METADATA_FILE_NAME,
    DATASET_METADATA_FILE_NAME,
]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import qwak.feature_store.features.feature_set_pb2
import qwak.feature_store.sources.data_source_pb2
import qwak.features_operator.v2.features_operator_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ValidateDataSourceRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCE_SPEC_FIELD_NUMBER: builtins.int
    NUM_SAMPLES_FIELD_NUMBER: builtins.int
    ZIP_PATH_FIELD_NUMBER: builtins.int
    @property
    def data_source_spec(self) -> qwak.feature_store.sources.data_source_pb2.DataSourceSpec:
        """Data source spec to validate"""
    num_samples: builtins.int
    """Number of samples to return from the validation"""
    zip_path: builtins.str
    """Dependencies zip file path"""
    def __init__(
        self,
        *,
        data_source_spec: qwak.feature_store.sources.data_source_pb2.DataSourceSpec | None = ...,
        num_samples: builtins.int = ...,
        zip_path: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["data_source_spec", b"data_source_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_source_spec", b"data_source_spec", "num_samples", b"num_samples", "zip_path", b"zip_path"]) -> None: ...

global___ValidateDataSourceRequest = ValidateDataSourceRequest

class ValidateFeatureSetRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_SET_SPEC_FIELD_NUMBER: builtins.int
    ZIP_PATH_FIELD_NUMBER: builtins.int
    NUM_SAMPLES_FIELD_NUMBER: builtins.int
    @property
    def feature_set_spec(self) -> qwak.feature_store.features.feature_set_pb2.FeatureSetSpec:
        """Featureset spec to validate"""
    zip_path: builtins.str
    """Dependencies zip file path"""
    num_samples: builtins.int
    """Number of samples to return from the validation"""
    def __init__(
        self,
        *,
        feature_set_spec: qwak.feature_store.features.feature_set_pb2.FeatureSetSpec | None = ...,
        zip_path: builtins.str = ...,
        num_samples: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["feature_set_spec", b"feature_set_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_set_spec", b"feature_set_spec", "num_samples", b"num_samples", "zip_path", b"zip_path"]) -> None: ...

global___ValidateFeatureSetRequest = ValidateFeatureSetRequest

class ValidationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_RESPONSE_FIELD_NUMBER: builtins.int
    FAILURE_RESPONSE_FIELD_NUMBER: builtins.int
    @property
    def success_response(self) -> qwak.features_operator.v2.features_operator_pb2.ValidationSuccessResponse: ...
    @property
    def failure_response(self) -> qwak.features_operator.v2.features_operator_pb2.ValidationFailureResponse: ...
    def __init__(
        self,
        *,
        success_response: qwak.features_operator.v2.features_operator_pb2.ValidationSuccessResponse | None = ...,
        failure_response: qwak.features_operator.v2.features_operator_pb2.ValidationFailureResponse | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["failure_response", b"failure_response", "success_response", b"success_response", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["failure_response", b"failure_response", "success_response", b"success_response", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["success_response", "failure_response"] | None: ...

global___ValidationResponse = ValidationResponse



from typing import Dict, List, Tuple

import grpc
from _qwak_proto.qwak.prompt.v1.prompt.prompt_manager_service_pb2 import (
    CreatePromptRequest as ProtoCreatePromptRequest,
    CreatePromptResponse as ProtoCreatePromptResponse,
    CreatePromptVersionRequest as ProtoCreatePromptVersionRequest,
    CreatePromptVersionResponse as ProtoCreatePromptVersionResponse,
    DeletePromptRequest as ProtoDeletePromptRequest,
    DeletePromptResponse as ProtoDeletePromptResponse,
    DeletePromptVersionRequest as ProtoDeletePromptVersionRequest,
    DeletePromptVersionResponse as ProtoDeletePromptVersionResponse,
    GetPromptByNameRequest as ProtoGetPromptByNameRequest,
    GetPromptByNameResponse as ProtoGetPromptByNameResponse,
    GetPromptVersionByPromptNameRequest as ProtoGetPromptVersionByPromptNameRequest,
    GetPromptVersionByPromptNameResponse as ProtoGetPromptVersionByPromptNameResponse,
    SetDefaultPromptVersionRequest as ProtoSetDefaultPromptVersionRequest,
    SetDefaultPromptVersionResponse as ProtoSetDefaultPromptVersionResponse,
    UpdatePromptRequest as ProtoUpdatePromptRequest,
    UpdatePromptResponse as ProtoUpdatePromptResponse,
)
from _qwak_proto.qwak.prompt.v1.prompt.prompt_manager_service_pb2_grpc import (
    PromptManagerServiceServicer,
)
from _qwak_proto.qwak.prompt.v1.prompt.prompt_pb2 import (
    Prompt as ProtoPrompt,
    PromptMetadata as ProtoPromptMetadata,
    PromptSpec as ProtoPromptSpec,
    PromptVersion as ProtoPromptVersion,
    PromptVersionDefinition as ProtoPromptVersionDefinition,
    PromptVersionMetadata as ProtoPromptVersionMetadata,
)
from google.protobuf.timestamp_pb2 import Timestamp


class PromptManagerServiceMock(PromptManagerServiceServicer):
    prompts: Dict[str, ProtoPrompt] = {}
    prompts_versions: Dict[Tuple[str, int], ProtoPromptVersionDefinition] = {}
    prompts_next_version: Dict[str, int] = {}

    def clear(self):
        self.prompts = {}
        self.prompts_versions = {}
        self.prompts_next_version = {}

    def _is_prompt_name_valid(self, test_str: str) -> bool:
        import re

        regex = r"^[a-z0-9](?:[-_]?[a-z0-9]+)+$"
        return re.match(pattern=regex, string=test_str) is not None

    def _raise_on_missing_prompt(self, prompt_name: str, context):
        if prompt_name not in self.prompts:
            context.set_code(grpc.StatusCode.NOT_FOUND)
            context.set_details("Prompt name does not exist")
            raise ValueError("Prompt name does not exist")

    def _raise_on_missing_prompt_version(self, prompt_name: str, version: int, context):
        if (
            prompt_name not in self.prompts
            or (prompt_name, version) not in self.prompts_versions
        ):
            context.set_code(grpc.StatusCode.NOT_FOUND)
            context.set_details("Prompt version does not exist")
            raise ValueError("Prompt version does not exist")

    def CreatePrompt(self, request: ProtoCreatePromptRequest, context):
        """Create a new prompt"""

        if request.prompt_name in self.prompts:
            context.set_code(grpc.StatusCode.ALREADY_EXISTS)
            context.set_details("Prompt name already exists")
            raise ValueError("Prompt name already exists")

        if not self._is_prompt_name_valid(request.prompt_name):
            context.set_code(grpc.StatusCode.INVALID_ARGUMENT)
            context.set_details("Bad Prompt name")
            raise ValueError("Bad Prompt name")

        prompt_version_definition = ProtoPromptVersionDefinition(
            version_number=1,
            version_spec=request.prompt_version_spec,
            version_metadata=ProtoPromptVersionMetadata(
                created_by="nirh@qwak.com", created_at=Timestamp().GetCurrentTime()
            ),
        )

        prompt = ProtoPrompt(
            name=request.prompt_name,
            default_version_definition=prompt_version_definition,
            metadata=ProtoPromptMetadata(
                created_at=Timestamp().GetCurrentTime(), created_by="nirh@qwak.com"
            ),
            prompt_spec=ProtoPromptSpec(description=request.prompt_spec.description),
        )

        self.prompts[prompt.name] = prompt
        self.prompts_next_version[prompt.name] = 2
        self.prompts_versions[
            (prompt.name, prompt.default_version_definition.version_number)
        ] = prompt_version_definition

        return ProtoCreatePromptResponse(prompt=prompt)

    def CreatePromptVersion(self, request: ProtoCreatePromptVersionRequest, context):
        """
        Create new prompt version.
        Set new version in versions dict.
        Bump prompt next version.
        Set default version if needed
        """
        self._raise_on_missing_prompt(prompt_name=request.prompt_name, context=context)

        existing_prompt: ProtoPrompt = self.prompts[request.prompt_name]
        new_prompt_version_number: int = self.prompts_next_version[request.prompt_name]
        prompt_version_definition = ProtoPromptVersionDefinition(
            version_number=new_prompt_version_number,
            version_spec=request.prompt_version_spec,
            version_metadata=ProtoPromptVersionMetadata(
                created_by="nirh@qwak.com", created_at=Timestamp().GetCurrentTime()
            ),
        )

        self.prompts_next_version[request.prompt_name] = new_prompt_version_number + 1

        if request.set_default:
            existing_prompt.default_version_definition.CopyFrom(
                prompt_version_definition
            )

        self.prompts_versions[
            (request.prompt_name, new_prompt_version_number)
        ] = prompt_version_definition
        return ProtoCreatePromptVersionResponse(
            prompt_version=ProtoPromptVersion(
                prompt_name=request.prompt_name,
                prompt_version_definition=prompt_version_definition,
            )
        )

    def UpdatePrompt(self, request: ProtoUpdatePromptRequest, context):
        """Update Prompt"""
        self._raise_on_missing_prompt(prompt_name=request.prompt_name, context=context)

        existing_prompt: ProtoPrompt = self.prompts[request.prompt_name]
        existing_prompt.prompt_spec.CopyFrom(request.prompt_spec)

        return ProtoUpdatePromptResponse()

    def GetPromptByName(self, request: ProtoGetPromptByNameRequest, context):
        """Get prompt with its default version by name"""
        self._raise_on_missing_prompt(prompt_name=request.prompt_name, context=context)

        existing_prompt: ProtoPrompt = self.prompts[request.prompt_name]
        return ProtoGetPromptByNameResponse(prompt=existing_prompt)

    def GetPromptVersionByPromptName(
        self, request: ProtoGetPromptVersionByPromptNameRequest, context
    ):
        """Get prompt version by name and optional version number"""

        self._raise_on_missing_prompt(
            prompt_name=request.prompt_name,
            context=context,
        )

        self._raise_on_missing_prompt_version(
            prompt_name=request.prompt_name,
            version=request.version_number,
            context=context,
        )

        return ProtoGetPromptVersionByPromptNameResponse(
            prompt_version=ProtoPromptVersion(
                prompt_name=request.prompt_name,
                prompt_version_definition=self.prompts_versions[
                    (request.prompt_name, request.version_number)
                ],
            )
        )

    def DeletePrompt(self, request: ProtoDeletePromptRequest, context):
        """Delete prompt"""
        self._raise_on_missing_prompt(prompt_name=request.prompt_name, context=context)

        del self.prompts[request.prompt_name]
        del self.prompts_next_version[request.prompt_name]

        versions_to_delete: List[int] = [
            version
            for (name, version) in self.prompts_versions.keys()
            if name == request.prompt_name
        ]

        for v in versions_to_delete:
            del self.prompts_versions[(request.prompt_name, v)]

        return ProtoDeletePromptResponse()

    def DeletePromptVersion(self, request: ProtoDeletePromptVersionRequest, context):
        """Delete prompt version"""
        self._raise_on_missing_prompt(
            prompt_name=request.prompt_name,
            context=context,
        )

        self._raise_on_missing_prompt_version(
            prompt_name=request.prompt_name,
            version=request.version_number,
            context=context,
        )

        prompt_default_version: ProtoPrompt = self.prompts[request.prompt_name]
        if (
            prompt_default_version.default_version_definition.version_number
            == request.version_number
        ):
            context.set_code(grpc.StatusCode.FAILED_PRECONDITION)
            context.set_details("Can't delete prompt default version")
            raise ValueError("Can't delete prompt default version")

        del self.prompts_versions[(request.prompt_name, request.version_number)]
        return ProtoDeletePromptVersionResponse()

    def SetDefaultPromptVersion(
        self, request: ProtoSetDefaultPromptVersionRequest, context
    ):
        """Set default prompt version"""
        self._raise_on_missing_prompt(
            prompt_name=request.prompt_name,
            context=context,
        )

        self._raise_on_missing_prompt_version(
            prompt_name=request.prompt_name,
            version=request.version_number,
            context=context,
        )
        existing_prompt: ProtoPrompt = self.prompts[request.prompt_name]
        prompt_version_definition: ProtoPromptVersionDefinition = self.prompts_versions[
            (request.prompt_name, request.version_number)
        ]
        existing_prompt.default_version_definition.CopyFrom(prompt_version_definition)
        return ProtoSetDefaultPromptVersionResponse()



import re
from dataclasses import dataclass
from typing import Dict, List, Optional, TypeVar, Union

from _qwak_proto.qwak.vectors.v1.vector_pb2 import (
    Property,
    SearchResult as ProtoSearchResult,
    StoredVector as ProtoStoredVector,
    VectorIdentifier as ProtoVectorIdentifier,
)
from qwak.clients.vector_store.serving_client import VectorServingClient
from qwak.exceptions import QwakException
from qwak.vector_store.filters import Filter
from qwak.vector_store.inference_client import VectorStoreInferenceClient
from qwak.vector_store.utils.upsert_utils import (
    _divide_chunks,
    _upsert_natural_input,
    _upsert_vectors,
)
from typeguard import typechecked

NaturalInput = TypeVar("T")
NaturalInputs = List[NaturalInput]
Vector = List[float]
Properties = Dict[str, Union[str, int, bool, float]]

_TENANT_ID_PATTERN = r"^[a-zA-Z0-9_-]{4,64}$"


@dataclass
class SearchResult:
    """
    A class used to represent the result of a vector similarity search operation.

     Attributes:
         properties (dict): The dictionary of properties to attach with the vectors
         id (str): The vector object unique identifier
         vector (Vector): The vector values
         distance (int): The distance metric indicating how similar the vector is to the search query.
             Smaller values indicate higher similarity.
    """

    properties: Properties
    id: Optional[str]
    vector: Optional[Vector]
    distance: Optional[float]


class Collection:
    """
    The Collection class is a Python class that provides functionalities for handling operations on vectors within a
    specific collection in a vector store. This class should be used after a collection has been created or fetched
    using `VectorStoreClient`.

    The Collection class allows you to:
        * **Search for Similar Vectors**: This helps in finding vectors that are most similar to a given query vector.
        * **Upsert Vectors**: This operation allows you to insert new vectors into the collection or update existing
            vectors if they already exist.
        * **Delete Vectors by ID**: This operation deletes vectors based on their unique identifiers
    """

    id: str
    name: str
    metric: str
    dimension: int
    description: Optional[str]
    vectorizer: Optional[str]
    multi_tenant: bool

    _vector_serving_client: VectorServingClient
    _type_to_proto_property_mapping: Dict[str, TypeVar] = {
        str: "string_val",
        bool: "bool_val",
        int: "int_val",
        float: "double_val",
    }

    _proto_property_to_type_mapping = {
        v: k for k, v in _type_to_proto_property_mapping.items()
    }

    def __init__(
        self,
        id: str,
        name: str,
        metric: str,
        dimension: int,
        vector_serving_client: VectorServingClient,
        description: Optional[str] = None,
        vectorizer: Optional[str] = None,
        muli_tenant: bool = False,
    ):
        """
        Initializes a `Collection` client object to interact with Qwak's vector serving service. Should not be created
        directly - created or fetched using the `VectorStoreClient` object.
        """
        self.id = id
        self.name = name
        self.description = description
        self.metric = metric
        self.dimension = dimension
        self.vectorizer = vectorizer
        self._vector_serving_client = vector_serving_client
        self._realtime_inference_client = None
        self.multi_tenant = muli_tenant

        if vectorizer:
            self._realtime_inference_client = VectorStoreInferenceClient(
                model_id=self.vectorizer.lower().replace(" ", "_").replace("-", "_")
            )

    @typechecked
    def search(
        self,
        output_properties: List[str],
        vector: Optional[Vector] = None,
        natural_input: Optional[NaturalInput] = None,
        top_results: int = 1,
        include_id: bool = True,
        include_vector: bool = False,
        include_distance: bool = False,
        filter: Optional[Filter] = None,
        tenant_id: Optional[str] = None,
    ) -> List[SearchResult]:
        """
        Searches for vectors in the collection that are most similar to a given query vector.
        Vector similarity is a measure of the closeness or similarity between two vectors. In the context of machine
        learning, vectors often represent points in a high-dimensional space, and the concept of similarity between
        vectors can be crucial for many tasks such as clustering, classification, and nearest-neighbor searches.

        Parameters:
            output_properties (list): A list of property fields to include in the results.
            vector (list): The vector to get the most similar vectors to according to the distance metric
            natural_input (any): Natural inputs (text, image) which should be embedded by the collection and, and
              according to the resulting embedding - get the most similar vectors
            top_results (int): The number of relevant results to return
            include_id (list): Whether to include the vector ID's in the result set
            include_vector (list): Whether to include the vector values themselves in the result set
            include_distance (list): Whether to include the distance calculations to the result set
            filter (Filter): Pre-filtering search results
            tenant_id (str): tenant ID, passed if and only if the collection has multi tenancy enabled

        Returns:
            List[SearchResult]: A list of SearchResult object, which is used as a container for the search results

        Raises:
            QwakException: If you don't provide either vectors or natural_inputs
            QwakException: If you provide both vectors and natural_inputs
            QwakException: If the tenant provided mismatches the configuration
        """
        if not (bool(vector) ^ bool(natural_input)):
            raise QwakException(
                "Exactly one of {'vectors', 'natural_input'} should be passed"
            )

        if natural_input:
            vector = self._transform_natural_input_to_vectors(
                natural_input=natural_input
            )
        proto_filter = filter._to_proto() if filter else None
        self._validate_tenant(tenant_id)

        return [
            self._to_search_result(
                result,
                include_id=include_id,
                include_distance=include_distance,
                include_vector=include_vector,
            )
            for result in self._vector_serving_client.search(
                collection_name=self.name,
                vector=vector,
                properties=output_properties,
                top_results=top_results,
                include_id=include_id,
                include_vector=include_vector,
                include_distance=include_distance,
                filters=proto_filter,
                tenant_id=tenant_id,
            )
        ]

    @typechecked
    def upsert(
        self,
        ids: List[str],
        properties: List[Properties],
        vectors: Optional[List[Vector]] = None,
        natural_inputs: Optional[NaturalInputs] = None,
        batch_size: int = 1000,
        multiproc: bool = False,
        max_processes: Optional[int] = None,
        *,
        tenant_ids: Optional[List[str]] = None,
    ) -> None:
        """
        Inserts new vectors into the collection or updates existing vectors. Notice that this method will overwrite
        existing vectors with the same IDs.

        Parameters:
            ids (str): A list of vector ids to be added
            vectors (list): The list of vectors to add. This attribute or `natural_inputs` must be set
            natural_inputs (list): Natural inputs (text, image) which should be embedded by the collection and added
              to the store. This attribute or `vectors` must be set
            properties (dict): A dictionary of properties to attach with the vectors
            batch_size(int): maximum batch size when upserting against the backend Vector Store, defaults to 1000
            multiproc (bool): whether to use multiprocessing, defaults to False
            max_processes (Optional[int]): max number of processes if multiproc is selected, multiproc must be enabled
            tenant_ids (List[str]): tenant ids, should be specified if and only if the collection has multi tenancy enabled.

        Raises:
            QwakException: If you don't provide either vectors or natural_inputs
            QwakException: If you provide both vectors and natural_inputs
        """

        if not (bool(vectors) ^ bool(natural_inputs)):
            raise QwakException(
                "`vectors` or `natural` inputs should be defined and not empty. But not both"
            )

        if max_processes is not None and not multiproc:
            raise QwakException("Can not set max_processes if multiproc is not enabled")

        self._validate_tenant_ids(
            vector_ids=vectors, tenant_ids=tenant_ids, verb="upserting"
        )
        id_tpls = zip(ids, tenant_ids) if self.multi_tenant else ids

        if (len(vectors or natural_inputs) != len(ids)) or (
            len(properties) != len(ids)
        ):
            raise QwakException(
                "Non matching lengths for input list (vectors / natural inputs), IDs, and properties list. "
                "Make sure all 3 fields are aligned in length"
            )
        if bool(natural_inputs):
            _upsert_natural_input(
                vector_tuples=list(zip(id_tpls, natural_inputs, properties)),
                chunk_size=batch_size,
                vectorizer_name=self.vectorizer,
                collection_name=self.name,
                edge_services_url=self._vector_serving_client._edge_services_url,
                multiproc=multiproc,
                max_processes=max_processes,
            )
        else:
            _upsert_vectors(
                vector_tuples=list(zip(id_tpls, vectors, properties)),
                chunk_size=batch_size,
                collection_name=self.name,
                edge_services_url=self._vector_serving_client._edge_services_url,
                multiproc=multiproc,
                max_processes=max_processes,
            )

    @typechecked
    def delete(
        self,
        vector_ids: List[str],
        *,
        tenant_ids: Optional[List[str]] = None,
        batch_size: int = 10000,
    ) -> int:
        """
        Deletes vectors from the collection based on their IDs.

        Parameters:
            vector_ids (list): A list of vector IDs to delete.
            batch_size (int): optional batch size, defaults to 10000
            tenant_ids (list): tenant IDs (same length as vector_ids, used only when multi tenancy is enabled)

        Returns:
            int: Number of actual vectors deleted from the collection
        """
        self._validate_tenant_ids(
            vector_ids=vector_ids, tenant_ids=tenant_ids, verb="deleting"
        )
        vector_identifiers: List[ProtoVectorIdentifier] = self._extract_tenant_ids(
            vector_ids, tenant_ids
        )

        return sum(
            self._vector_serving_client.delete_vectors(
                collection_name=self.name, vector_identifiers=ids_chunk
            )
            for ids_chunk in _divide_chunks(vector_identifiers, batch_size)
        )

    @typechecked
    def fetch(self, vector_id: str, *, tenant_id: Optional[str] = None) -> SearchResult:
        """
        Fetches a vector from the collection based on its ID.

        Parameters:
            vector_id (str): The ID of the vector to fetch.
            tenant_id (str, optional): Tenant id, passed if and only if multi tenancy is enabled

        Returns:
            SearchResult: A SearchResult object, which is used as a container for the search results
        """
        self._validate_tenant(tenant_id)
        vector_identifier: ProtoVectorIdentifier
        if tenant_id is not None:
            vector_identifier = ProtoVectorIdentifier(
                vector_id=vector_id, tenant_id=tenant_id
            )
        else:
            vector_identifier = ProtoVectorIdentifier(vector_id=vector_id)

        result = self._vector_serving_client.fetch_vector(
            collection_name=self.name, vector_identifier=vector_identifier
        )

        return self._to_search_result(
            result, include_id=True, include_distance=False, include_vector=True
        )

    def _to_search_result(
        self,
        search_result: Union[ProtoSearchResult, ProtoStoredVector],
        include_id: bool,
        include_vector: bool,
        include_distance: bool,
    ) -> SearchResult:
        id = (
            search_result.vector_identifier.vector_id
            if type(search_result) is ProtoStoredVector
            else search_result.id
        )
        return SearchResult(
            id=id if include_id else None,
            vector=(
                [e for e in search_result.vector.element] if include_vector else None
            ),
            distance=search_result.distance if include_distance else None,
            properties={
                prop.name: self._extract_value_with_type(prop)
                for prop in search_result.properties
            },
        )

    def _extract_value_with_type(self, prop: Property):
        type_caster = self._proto_property_to_type_mapping.get(
            prop.WhichOneof("value_type"), None
        )
        if not type_caster:
            raise QwakException(
                f"Cannot deserialize property with type {type(type_caster)}. This means an invalid property type"
                f" was registered to the platform. Please delete and add the vector object again."
            )

        return type_caster(getattr(prop, prop.WhichOneof("value_type")))

    def _transform_natural_input_to_vectors(
        self, natural_input: NaturalInput
    ) -> Vector:
        if not self.vectorizer:
            raise QwakException(
                "Unable to search by natural input because the collection does not have a Vectorizer defined."
            )
        return self._realtime_inference_client.get_embedding(natural_input)

    def _transform_natural_input_list_to_vectors(
        self, natural_inputs: NaturalInputs
    ) -> List[Vector]:
        return [
            self._transform_natural_input_to_vectors(natural_input=natural_input)
            for natural_input in natural_inputs
        ]

    def _validate_tenant(self, tenant_id: Optional[str] = None):
        if self.multi_tenant:
            # we are multi tenant, assert a valid tenant is passed
            if tenant_id is None:
                raise QwakException(
                    "Tenant ID must be passed when multi tenancy is enabled"
                )

            self._validate_tenant_id(tenant_id)
        else:
            if tenant_id is not None:
                raise QwakException(
                    f"Collection {self.name} is not multi tenant, can not specify tenant"
                )

    def _validate_tenant_id(self, tenant_id: str):
        if not (bool(re.match(_TENANT_ID_PATTERN, tenant_id))):
            raise QwakException(
                f"Tenant ID {tenant_id} does not conform to {_TENANT_ID_PATTERN}"
            )

    def _validate_tenant_ids(
        self, vector_ids: List[str], tenant_ids: Optional[List[str]], verb: str
    ) -> None:
        if self.multi_tenant:
            if tenant_ids is None:
                raise QwakException(
                    f"Tenant IDs must be provided when {verb} against multitenant collections"
                )
            if len(tenant_ids) != len(vector_ids):
                raise QwakException(
                    f"Got different number of vector ids {len(vector_ids)} and tenant ids {len(tenant_ids)}"
                )
            for tenant_id in tenant_ids:
                self._validate_tenant_id(tenant_id=tenant_id)
        else:
            if tenant_ids is not None:
                raise QwakException(
                    f"Collection {self.name} does not have multi tenancy enabled, do not pass tenant ids"
                )

    def _extract_tenant_ids(
        self, vector_ids: List[str], tenant_ids: Optional[List[str]]
    ) -> List[ProtoVectorIdentifier]:
        vector_identifiers: List[ProtoVectorIdentifier]
        if self.multi_tenant:
            vector_identifiers = [
                ProtoVectorIdentifier(vector_id=vector_id, tenant_id=tenant_id)
                for vector_id, tenant_id in zip(vector_ids, tenant_ids)
            ]
        else:
            vector_identifiers = [
                ProtoVectorIdentifier(vector_id=vector_id) for vector_id in vector_ids
            ]

        return vector_identifiers



from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum

from _qwak_proto.qwak.automation.v1.automation_execution_pb2 import (
    AutomationExecutionMessage,
    ExecutionRunDetailsMessage,
    ExecutionStatusMessage,
)
from google.protobuf.timestamp_pb2 import Timestamp


class ExecutionStatus(Enum):
    UNDEFINED = 0
    RUNNING = 1
    SUCCESSFUL = 2
    FAILED = 3


execution_status_to_proto_map = {
    ExecutionStatus.UNDEFINED: ExecutionStatusMessage.UNDEFINED,
    ExecutionStatus.RUNNING: ExecutionStatusMessage.RUNNING,
    ExecutionStatus.FAILED: ExecutionStatusMessage.FAILED,
    ExecutionStatus.SUCCESSFUL: ExecutionStatusMessage.SUCCESSFUL,
}

proto_status_to_execution_status_map = {
    v: k for k, v in execution_status_to_proto_map.items()
}


@dataclass
class ExecutionRunDetails:
    start_time: datetime = field(default=None)
    end_time: datetime = field(default=None)
    error_details: str = field(default="")
    status: ExecutionStatus = field(default=ExecutionStatus.UNDEFINED)
    task: str = field(default="")
    finish_cause: str = field(default="")

    def to_proto(self):
        start_time_timestamp = None
        end_time_timestamp = None
        if self.start_time:
            start_time_timestamp = Timestamp()
            start_time_timestamp.FromDatetime(self.start_time)

        if self.end_time:
            end_time_timestamp = Timestamp()
            end_time_timestamp.FromDatetime(self.end_time)

        return ExecutionRunDetailsMessage(
            start_time=start_time_timestamp,
            end_time=end_time_timestamp,
            error_details=self.error_details,
            status=execution_status_to_proto_map.get(self.status),
            task=self.task,
            finish_cause=self.finish_cause,
        )

    @staticmethod
    def from_proto(message: ExecutionRunDetailsMessage):
        return ExecutionRunDetails(
            start_time=(
                datetime.fromtimestamp(
                    message.start_time.seconds + message.start_time.nanos / 1e9
                )
                if message.start_time
                else None
            ),
            end_time=(
                datetime.fromtimestamp(
                    message.end_time.seconds + message.end_time.nanos / 1e9
                )
                if message.end_time
                else None
            ),
            status=proto_status_to_execution_status_map.get(message.status),
            task=message.task,
            error_details=message.error_details,
            finish_cause=message.finish_cause,
        )


@dataclass
class AutomationExecution:
    execution_id: str = field(default="")
    run_details: ExecutionRunDetails = field(default_factory=ExecutionRunDetails)

    @staticmethod
    def from_proto(message: AutomationExecutionMessage):
        return AutomationExecution(
            execution_id=message.execution_id,
            run_details=ExecutionRunDetails.from_proto(message.run_details),
        )

    def __str__(self):
        return f"Id: {self.execution_id}\nDetails:\nStart Time:\t{self.run_details.start_time}\nEnd Time:\t{self.run_details.end_time}\nStatus:\t{self.run_details.status.name}\n"



import os
import re
from typing import List, Optional, Union

from frogml_storage.logging import logger
from frogml_storage.constants import FROG_ML_MAX_CHARS_FOR_NAME
from frogml_storage.exceptions.validation_error import FrogMLValidationError

valid_characters_pattern = re.compile(r"^[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)*$")


def user_input_validation(
    entity_name: Optional[str],
    namespace: Optional[str],
    version: Optional[str],
    properties: Optional[dict[str, str]] = None,
) -> bool:
    for arg_name, arg_value in {
        "entity_name": entity_name,
        "namespace": namespace,
        "version": version,
    }.items():
        if arg_value is not None:
            __input_validation(arg_value, arg_name)

    if properties is not None:
        __user_input_dict_validation(properties)

    return True


def __user_input_dict_validation(properties: dict[str, str]) -> bool:
    if properties is not None:
        for key, value in properties.items():
            __input_validation(key, "properties")
            __input_validation(value, "properties")
    return True


def is_not_none(arg_name: str, arg_value: Optional[object]) -> bool:
    if arg_value is None:
        raise FrogMLValidationError("{} can't be 'None'.".format(arg_name))
    return True


def __input_validation(field_value: str, field_name: str) -> bool:
    if len(str(field_value)) > FROG_ML_MAX_CHARS_FOR_NAME:
        raise FrogMLValidationError(
            "Max length for {} is 60 characters.".format(field_name.capitalize())
        )

    if not field_value or not re.match(valid_characters_pattern, str(field_value)):
        raise FrogMLValidationError(
            "Invalid characters detected at {}: {}".format(
                field_name.capitalize(), field_value
            )
        )

    return True


def is_valid_thread_number(thread_count: str) -> bool:
    try:
        int_thread_count = int(thread_count)
        cpu_count = os.cpu_count()
        if int_thread_count <= 0 or (
            cpu_count is not None and int_thread_count >= cpu_count
        ):
            raise ValueError(
                "Invalid thread count: {}. The default value will be used.".format(
                    thread_count
                )
            )
        return True
    except ValueError as e:
        logger.warning("Thread count {}: {}".format(thread_count, e))
        return False
    except TypeError:
        logger.debug("Thread count not configured. The default value will be used.")
        return False


def validate_not_folder_paths(paths: Union[Optional[List[str]], Optional[str]]) -> bool:
    if paths is not None:
        if isinstance(paths, List):
            for path in paths:
                __validate_not_folder_path(path)
        else:
            __validate_not_folder_path(paths)
    return True


def __validate_not_folder_path(path: str) -> bool:
    if os.path.isdir(path):
        raise FrogMLValidationError(
            "file '{}' must be a file, but is a directory.".format(path)
        )
    return True


def validate_path_exists(path: str) -> bool:
    if not os.path.exists(path):
        raise ValueError(f"Provided path does not exists : '{path}'")
    return True



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import qwak.administration.runtime_configuration.v0.external.redis_config_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class FeatureStoreConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENABLED_FIELD_NUMBER: builtins.int
    @property
    def enabled(self) -> global___EnabledFeatureStoreConfiguration: ...
    def __init__(
        self,
        *,
        enabled: global___EnabledFeatureStoreConfiguration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["enabled", b"enabled", "toggle", b"toggle"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["enabled", b"enabled", "toggle", b"toggle"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["toggle", b"toggle"]) -> typing_extensions.Literal["enabled"] | None: ...

global___FeatureStoreConfiguration = FeatureStoreConfiguration

class EnabledFeatureStoreConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ONLINE_FEATURE_STORE_FIELD_NUMBER: builtins.int
    @property
    def online_feature_store(self) -> global___OnlineFeatureStore: ...
    def __init__(
        self,
        *,
        online_feature_store: global___OnlineFeatureStore | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["online_feature_store", b"online_feature_store"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["online_feature_store", b"online_feature_store"]) -> None: ...

global___EnabledFeatureStoreConfiguration = EnabledFeatureStoreConfiguration

class OnlineFeatureStore(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REDIS_CONFIGURATION_FIELD_NUMBER: builtins.int
    @property
    def redis_configuration(self) -> qwak.administration.runtime_configuration.v0.external.redis_config_pb2.RedisConfiguration: ...
    def __init__(
        self,
        *,
        redis_configuration: qwak.administration.runtime_configuration.v0.external.redis_config_pb2.RedisConfiguration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["redis_configuration", b"redis_configuration", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["redis_configuration", b"redis_configuration", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["redis_configuration"] | None: ...

global___OnlineFeatureStore = OnlineFeatureStore



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/analytics/analytics.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1eqwak/analytics/analytics.proto\x12\x0eqwak.analytics\"w\n\nResultData\x12\x0f\n\x07headers\x18\x01 \x03(\t\x12/\n\x0bheadersDesc\x18\x03 \x03(\x0b\x32\x1a.qwak.analytics.HeaderDesc\x12\'\n\x04rows\x18\x02 \x03(\x0b\x32\x19.qwak.analytics.ResultRow\"(\n\nHeaderDesc\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\"\x1b\n\tResultRow\x12\x0e\n\x06values\x18\x01 \x03(\t\":\n\x0bTableColumn\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x0f\n\x07\x63omment\x18\x03 \x01(\t\"O\n\x15TableColumnDefinition\x12\x0c\n\x04name\x18\x01 \x01(\t\x12(\n\x04type\x18\x02 \x01(\x0e\x32\x1a.qwak.analytics.ColumnType\"\xe8\x01\n\x0fTableDefinition\x12\x0f\n\x07\x64\x62_name\x18\x01 \x01(\t\x12\x12\n\ntable_name\x18\x02 \x01(\t\x12\x36\n\x07\x63olumns\x18\x03 \x03(\x0b\x32%.qwak.analytics.TableColumnDefinition\x12\x11\n\tdata_path\x18\x04 \x01(\t\x12*\n\x06\x66ormat\x18\x05 \x01(\x0b\x32\x1a.qwak.analytics.DataFormat\x12\x39\n\npartitions\x18\x06 \x03(\x0b\x32%.qwak.analytics.TableColumnDefinition\"?\n\nDataFormat\x12\'\n\x03\x63sv\x18\x01 \x01(\x0b\x32\x18.qwak.analytics.CsvInputH\x00\x42\x08\n\x06\x66ormat\"F\n\x08\x43svInput\x12\x11\n\tdelimiter\x18\x01 \x01(\t\x12\x12\n\nquote_char\x18\x02 \x01(\t\x12\x13\n\x0b\x65scape_char\x18\x03 \x01(\t\"0\n\x1cQueryResultDownloadURLParams\x12\x10\n\x08query_id\x18\x01 \x01(\t*N\n\x0bQueryStatus\x12\x0b\n\x07INVALID\x10\x00\x12\x0b\n\x07SUCCESS\x10\x01\x12\x0b\n\x07PENDING\x10\x02\x12\x0c\n\x08\x43\x41NCELED\x10\x03\x12\n\n\x06\x46\x41ILED\x10\x04*\xad\x01\n\nColumnType\x12\x10\n\x0cINVALID_TYPE\x10\x00\x12\x0b\n\x07\x42OOLEAN\x10\x01\x12\x0b\n\x07TINYINT\x10\x02\x12\x0c\n\x08SMALLINT\x10\x03\x12\x07\n\x03INT\x10\x04\x12\n\n\x06\x42IGINT\x10\x05\x12\n\n\x06\x44OUBLE\x10\x06\x12\t\n\x05\x46LOAT\x10\x07\x12\x08\n\x04\x43HAR\x10\x08\x12\n\n\x06STRING\x10\t\x12\n\n\x06\x42INARY\x10\n\x12\x08\n\x04\x44\x41TE\x10\x0b\x12\r\n\tTIMESTAMP\x10\x0c\x42\x86\x01\n\x19\x63om.qwak.ai.analytics.apiP\x01Zggithub.com/qwak-ai/qwak-platform/services/core/java/analytics/analytics-api/pb/qwak/analytics;analyticsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.analytics.analytics_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031com.qwak.ai.analytics.apiP\001Zggithub.com/qwak-ai/qwak-platform/services/core/java/analytics/analytics-api/pb/qwak/analytics;analytics'
  _globals['_QUERYSTATUS']._serialized_start=805
  _globals['_QUERYSTATUS']._serialized_end=883
  _globals['_COLUMNTYPE']._serialized_start=886
  _globals['_COLUMNTYPE']._serialized_end=1059
  _globals['_RESULTDATA']._serialized_start=50
  _globals['_RESULTDATA']._serialized_end=169
  _globals['_HEADERDESC']._serialized_start=171
  _globals['_HEADERDESC']._serialized_end=211
  _globals['_RESULTROW']._serialized_start=213
  _globals['_RESULTROW']._serialized_end=240
  _globals['_TABLECOLUMN']._serialized_start=242
  _globals['_TABLECOLUMN']._serialized_end=300
  _globals['_TABLECOLUMNDEFINITION']._serialized_start=302
  _globals['_TABLECOLUMNDEFINITION']._serialized_end=381
  _globals['_TABLEDEFINITION']._serialized_start=384
  _globals['_TABLEDEFINITION']._serialized_end=616
  _globals['_DATAFORMAT']._serialized_start=618
  _globals['_DATAFORMAT']._serialized_end=681
  _globals['_CSVINPUT']._serialized_start=683
  _globals['_CSVINPUT']._serialized_end=753
  _globals['_QUERYRESULTDOWNLOADURLPARAMS']._serialized_start=755
  _globals['_QUERYRESULTDOWNLOADURLPARAMS']._serialized_end=803
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from typing import List, Optional

from qwak.inner.build_logic.interface.build_logger_interface import BuildLogger

from qwak.exceptions import QwakSuggestionException
from .common import is_git_uri, is_local_dir, is_zip_uri
from .strategy.folder.folder_strategy import FolderStrategy
from .strategy.git.git_strategy import GitStrategy
from .strategy.strategy import Strategy
from .strategy.zip.zip_strategy import ZipStrategy

URI_CHECK_STRATEGY_MAPPING = {
    is_local_dir: FolderStrategy,
    is_git_uri: GitStrategy,
    is_zip_uri: ZipStrategy,
}


class FetchStrategyManager:
    def __init__(self, uri: str, build_logger: BuildLogger) -> None:
        self._strategy: Optional[Strategy] = None
        self._uri = uri
        for check, clazz in URI_CHECK_STRATEGY_MAPPING.items():
            if check(uri):
                self._strategy = clazz(build_logger)

        if self._strategy is None:
            raise QwakSuggestionException(
                message=f"Model URI {uri} is no valid.",
                suggestion=f"Please make sure that model path: {uri} is a zip file/git uri/local folder.",
            )

    def fetch(
        self,
        dest: str,
        git_branch: str,
        git_credentials: str,
        model_id: str,
        build_id: str,
        custom_dependencies_path: str,
        main_dir: str,
        dependency_path: str,
        lock_dependency_path: str,
        dependency_required_folders: List[str],
        git_ssh_key: str,
    ) -> str:
        return self._strategy.fetch(
            src=self._uri,
            dest=dest,
            git_branch=git_branch,
            git_credentials=git_credentials,
            model_id=model_id,
            build_id=build_id,
            custom_dependencies_path=custom_dependencies_path,
            main_dir=main_dir,
            dependency_path=dependency_path,
            lock_dependency_path=lock_dependency_path,
            dependency_required_folders=dependency_required_folders,
            git_ssh_key=git_ssh_key,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/administration/runtime_configuration/v0/hosting_config.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.administration.runtime_configuration.v0.creds import secret_pb2 as qwak_dot_administration_dot_runtime__configuration_dot_v0_dot_creds_dot_secret__pb2
from _qwak_proto.qwak.administration.runtime_configuration.v0.hosting.aws import auth_pb2 as qwak_dot_administration_dot_runtime__configuration_dot_v0_dot_hosting_dot_aws_dot_auth__pb2
from _qwak_proto.qwak.administration.runtime_configuration.v0.hosting.azure import auth_pb2 as qwak_dot_administration_dot_runtime__configuration_dot_v0_dot_hosting_dot_azure_dot_auth__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAqwak/administration/runtime_configuration/v0/hosting_config.proto\x12;qwak.administration.runtime_configuration.v0.hosting_config\x1a?qwak/administration/runtime_configuration/v0/creds/secret.proto\x1a\x43qwak/administration/runtime_configuration/v0/hosting/aws/auth.proto\x1a\x45qwak/administration/runtime_configuration/v0/hosting/azure/auth.proto\"\x8a\x03\n\x14HostingConfiguration\x12v\n\x16\x61ws_host_configuration\x18\x01 \x01(\x0b\x32T.qwak.administration.runtime_configuration.v0.hosting_config.AWSHostingConfigurationH\x00\x12v\n\x16gcp_host_configuration\x18\x02 \x01(\x0b\x32T.qwak.administration.runtime_configuration.v0.hosting_config.GCPHostingConfigurationH\x00\x12z\n\x18\x61zure_host_configuration\x18\x03 \x01(\x0b\x32V.qwak.administration.runtime_configuration.v0.hosting_config.AzureHostingConfigurationH\x00\x42\x06\n\x04type\"\x8e\x02\n\x17\x41WSHostingConfiguration\x12m\n\x13iam_service_account\x18\x01 \x01(\x0b\x32N.qwak.administration.runtime_configuration.v0.hosting_config.IAMServiceAccountH\x00\x12l\n\x12scoped_permissions\x18\x03 \x01(\x0b\x32N.qwak.administration.runtime_configuration.v0.hosting.aws.AWSScopedPermissionsH\x00\x12\x0e\n\x06region\x18\x02 \x01(\tB\x06\n\x04\x61uth\"\xb8\x01\n\x17GCPHostingConfiguration\x12q\n\x13gcp_service_account\x18\x01 \x01(\x0b\x32R.qwak.administration.runtime_configuration.v0.hosting_config.GCPServiceAccountAuthH\x00\x12\x0e\n\x06region\x18\x02 \x01(\t\x12\x12\n\nproject_id\x18\x03 \x01(\tB\x06\n\x04\x61uth\"\xe7\x01\n\x15GCPServiceAccountAuth\x12g\n\x0fservice_account\x18\x01 \x01(\x0b\x32N.qwak.administration.runtime_configuration.v0.hosting_config.IAMServiceAccount\x12\x65\n\x14service_account_json\x18\x02 \x01(\x0b\x32G.qwak.administration.runtime_configuration.v0.creds.secret.SecretKeyRef\"&\n\x11IAMServiceAccount\x12\x11\n\trole_name\x18\x01 \x01(\t\"\xa9\x01\n\x19\x41zureHostingConfiguration\x12\x63\n\nazure_auth\x18\x01 \x01(\x0b\x32O.qwak.administration.runtime_configuration.v0.hosting.azure.AzureAuthentication\x12\x0e\n\x06region\x18\x02 \x01(\t\x12\x17\n\x0f\x61zure_tenant_id\x18\x03 \x01(\tB\xdb\x01\n4com.jfrog.ml.runtime_configuration.v0.hosting_configP\x01Z\xa0\x01github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/runtime_configuration/v0;runtime_configuration_v0b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.administration.runtime_configuration.v0.hosting_config_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n4com.jfrog.ml.runtime_configuration.v0.hosting_configP\001Z\240\001github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/runtime_configuration/v0;runtime_configuration_v0'
  _globals['_HOSTINGCONFIGURATION']._serialized_start=336
  _globals['_HOSTINGCONFIGURATION']._serialized_end=730
  _globals['_AWSHOSTINGCONFIGURATION']._serialized_start=733
  _globals['_AWSHOSTINGCONFIGURATION']._serialized_end=1003
  _globals['_GCPHOSTINGCONFIGURATION']._serialized_start=1006
  _globals['_GCPHOSTINGCONFIGURATION']._serialized_end=1190
  _globals['_GCPSERVICEACCOUNTAUTH']._serialized_start=1193
  _globals['_GCPSERVICEACCOUNTAUTH']._serialized_end=1424
  _globals['_IAMSERVICEACCOUNT']._serialized_start=1426
  _globals['_IAMSERVICEACCOUNT']._serialized_end=1464
  _globals['_AZUREHOSTINGCONFIGURATION']._serialized_start=1467
  _globals['_AZUREHOSTINGCONFIGURATION']._serialized_end=1636
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class SlackAppIntegration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENCRYPTED_SLACK_APP_AUTH_CONFIG_FIELD_NUMBER: builtins.int
    SLACK_APP_SETTINGS_FIELD_NUMBER: builtins.int
    SLACK_SYSTEM_SECRET_FIELD_NUMBER: builtins.int
    @property
    def encrypted_slack_app_auth_config(self) -> global___SlackAppAuthConfig:
        """Encrypted Slack auth configuration"""
    @property
    def slack_app_settings(self) -> global___SlackAppSettings:
        """All Qwak Slack App settings"""
    @property
    def slack_system_secret(self) -> global___SlackSystemSecretDescriptor:
        """Slack system secret descriptor"""
    def __init__(
        self,
        *,
        encrypted_slack_app_auth_config: global___SlackAppAuthConfig | None = ...,
        slack_app_settings: global___SlackAppSettings | None = ...,
        slack_system_secret: global___SlackSystemSecretDescriptor | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["auth", b"auth", "encrypted_slack_app_auth_config", b"encrypted_slack_app_auth_config", "slack_app_settings", b"slack_app_settings", "slack_system_secret", b"slack_system_secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["auth", b"auth", "encrypted_slack_app_auth_config", b"encrypted_slack_app_auth_config", "slack_app_settings", b"slack_app_settings", "slack_system_secret", b"slack_system_secret"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["auth", b"auth"]) -> typing_extensions.Literal["slack_system_secret"] | None: ...

global___SlackAppIntegration = SlackAppIntegration

class SlackAppIntegrationSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SLACK_APP_TEMPORARY_CREDENTIALS_FIELD_NUMBER: builtins.int
    @property
    def slack_app_temporary_credentials(self) -> global___SlackAppTemporaryCredentials: ...
    def __init__(
        self,
        *,
        slack_app_temporary_credentials: global___SlackAppTemporaryCredentials | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["credentials", b"credentials", "slack_app_temporary_credentials", b"slack_app_temporary_credentials"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["credentials", b"credentials", "slack_app_temporary_credentials", b"slack_app_temporary_credentials"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["credentials", b"credentials"]) -> typing_extensions.Literal["slack_app_temporary_credentials"] | None: ...

global___SlackAppIntegrationSpec = SlackAppIntegrationSpec

class SlackAppTemporaryCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STATE_FIELD_NUMBER: builtins.int
    CODE_FIELD_NUMBER: builtins.int
    REDIRECT_URI_FIELD_NUMBER: builtins.int
    state: builtins.str
    """A verification phrase generated by the app owner and required in order to validate the request"""
    code: builtins.str
    """A temporary Oauth2 token from slack"""
    redirect_uri: builtins.str
    """The redirect uri defined in the oauth url code"""
    def __init__(
        self,
        *,
        state: builtins.str = ...,
        code: builtins.str = ...,
        redirect_uri: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["code", b"code", "redirect_uri", b"redirect_uri", "state", b"state"]) -> None: ...

global___SlackAppTemporaryCredentials = SlackAppTemporaryCredentials

class SlackAppSettings(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SLACK_TEAM_FIELD_NUMBER: builtins.int
    SLACK_ENTERPRISE_FIELD_NUMBER: builtins.int
    @property
    def slack_team(self) -> global___SlackTeam:
        """Slack team spec"""
    @property
    def slack_enterprise(self) -> global___SlackEnterprise: ...
    def __init__(
        self,
        *,
        slack_team: global___SlackTeam | None = ...,
        slack_enterprise: global___SlackEnterprise | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["enterprise", b"enterprise", "slack_enterprise", b"slack_enterprise", "slack_team", b"slack_team"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["enterprise", b"enterprise", "slack_enterprise", b"slack_enterprise", "slack_team", b"slack_team"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["enterprise", b"enterprise"]) -> typing_extensions.Literal["slack_enterprise"] | None: ...

global___SlackAppSettings = SlackAppSettings

class SlackAppAuthConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BOT_USER_TOKEN_FIELD_NUMBER: builtins.int
    bot_user_token: builtins.str
    def __init__(
        self,
        *,
        bot_user_token: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["bot_user_token", b"bot_user_token"]) -> None: ...

global___SlackAppAuthConfig = SlackAppAuthConfig

class SlackSystemSecretDescriptor(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_NAME_FIELD_NUMBER: builtins.int
    URL_KEY_FIELD_NUMBER: builtins.int
    TOKEN_KEY_FIELD_NUMBER: builtins.int
    secret_name: builtins.str
    """Slack system secret name"""
    url_key: builtins.str
    """Slack system secret url key"""
    token_key: builtins.str
    """Slack system secret token key"""
    def __init__(
        self,
        *,
        secret_name: builtins.str = ...,
        url_key: builtins.str = ...,
        token_key: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret_name", b"secret_name", "token_key", b"token_key", "url_key", b"url_key"]) -> None: ...

global___SlackSystemSecretDescriptor = SlackSystemSecretDescriptor

class SlackAppValidationSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___SlackAppValidationSpec = SlackAppValidationSpec

class SlackAppIntegrationOptions(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___SlackAppIntegrationOptions = SlackAppIntegrationOptions

class SlackTeam(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Slack team id"""
    name: builtins.str
    """Slack team name"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id", "name", b"name"]) -> None: ...

global___SlackTeam = SlackTeam

class SlackEnterprise(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Slack enterprise id"""
    name: builtins.str
    """Slack enterprise name"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id", "name", b"name"]) -> None: ...

global___SlackEnterprise = SlackEnterprise



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/offline/serving/v1/options.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%qwak/offline/serving/v1/options.proto\x12%qwak.feature.store.offline.serving.v1\"G\n\x1aOfflineServingQueryOptions\x12)\n!include_featureset_version_column\x18\x01 \x01(\x08\x42&\n\"com.qwak.ai.offline.serving.api.v1P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.offline.serving.v1.options_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\"com.qwak.ai.offline.serving.api.v1P\001'
  _globals['_OFFLINESERVINGQUERYOPTIONS']._serialized_start=80
  _globals['_OFFLINESERVINGQUERYOPTIONS']._serialized_end=151
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/data_versioning/data_versioning_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.data_versioning import data_versioning_pb2 as qwak_dot_data__versioning_dot_data__versioning__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2qwak/data_versioning/data_versioning_service.proto\x12\x1fqwak.management.data_versioning\x1a*qwak/data_versioning/data_versioning.proto\"]\n\x16RegisterDataTagRequest\x12\x43\n\rdata_tag_spec\x18\x01 \x01(\x0b\x32,.qwak.management.data_versioning.DataTagSpec\"\x19\n\x17RegisterDataTagResponse\"}\n\x17GetModelDataTagsRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x10\n\x08\x62uild_id\x18\x02 \x01(\t\x12>\n\x06\x66ilter\x18\x03 \x01(\x0b\x32..qwak.management.data_versioning.DataTagFilter\"[\n\x18GetModelDataTagsResponse\x12?\n\tdata_tags\x18\x01 \x03(\x0b\x32,.qwak.management.data_versioning.DataTagSpec2\xb2\x02\n\x1f\x44\x61taVersioningManagementService\x12\x84\x01\n\x0fRegisterDataTag\x12\x37.qwak.management.data_versioning.RegisterDataTagRequest\x1a\x38.qwak.management.data_versioning.RegisterDataTagResponse\x12\x87\x01\n\x10GetModelDataTags\x12\x38.qwak.management.data_versioning.GetModelDataTagsRequest\x1a\x39.qwak.management.data_versioning.GetModelDataTagsResponseB/\n+com.qwak.ai.management.model_versioning.apiP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.data_versioning.data_versioning_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n+com.qwak.ai.management.model_versioning.apiP\001'
  _globals['_REGISTERDATATAGREQUEST']._serialized_start=131
  _globals['_REGISTERDATATAGREQUEST']._serialized_end=224
  _globals['_REGISTERDATATAGRESPONSE']._serialized_start=226
  _globals['_REGISTERDATATAGRESPONSE']._serialized_end=251
  _globals['_GETMODELDATATAGSREQUEST']._serialized_start=253
  _globals['_GETMODELDATATAGSREQUEST']._serialized_end=378
  _globals['_GETMODELDATATAGSRESPONSE']._serialized_start=380
  _globals['_GETMODELDATATAGSRESPONSE']._serialized_end=471
  _globals['_DATAVERSIONINGMANAGEMENTSERVICE']._serialized_start=474
  _globals['_DATAVERSIONINGMANAGEMENTSERVICE']._serialized_end=780
# @@protoc_insertion_point(module_scope)



from typing import Dict, Optional, Union

try:
    from bentoml.exceptions import InferenceException
except ImportError:
    from .qwak_mock_http_exception import MockHttpException as InferenceException


class QwakHTTPException(InferenceException):
    def __init__(
        self,
        status_code: int,
        message: Union[str, Dict],
        exception_class_name: Optional[str] = None,
    ):
        super().__init__(status_code, message)
        self.exception_class_name = (
            exception_class_name if exception_class_name else type(self).__name__
        )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _BuildVersioningTagsType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _BuildVersioningTagsTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_BuildVersioningTagsType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID_TAG_TYPE: _BuildVersioningTagsType.ValueType  # 0
    DATA_TAG_TYPE: _BuildVersioningTagsType.ValueType  # 1
    FILE_TAG_TYPE: _BuildVersioningTagsType.ValueType  # 2
    CODE_TAG_TYPE: _BuildVersioningTagsType.ValueType  # 3

class BuildVersioningTagsType(_BuildVersioningTagsType, metaclass=_BuildVersioningTagsTypeEnumTypeWrapper):
    """Tags Types"""

INVALID_TAG_TYPE: BuildVersioningTagsType.ValueType  # 0
DATA_TAG_TYPE: BuildVersioningTagsType.ValueType  # 1
FILE_TAG_TYPE: BuildVersioningTagsType.ValueType  # 2
CODE_TAG_TYPE: BuildVersioningTagsType.ValueType  # 3
global___BuildVersioningTagsType = BuildVersioningTagsType

class BuildVersioningUrlParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_ID_FIELD_NUMBER: builtins.int
    BUILD_ID_FIELD_NUMBER: builtins.int
    TAG_FIELD_NUMBER: builtins.int
    ENVIRONMENT_ID_FIELD_NUMBER: builtins.int
    TAG_TYPE_FIELD_NUMBER: builtins.int
    USE_CLOUD_PROVIDER_STORAGE_FIELD_NUMBER: builtins.int
    model_id: builtins.str
    """Model ID"""
    build_id: builtins.str
    """Build Id"""
    tag: builtins.str
    """Tag to save under"""
    environment_id: builtins.str
    """Environment to save the artifact under"""
    tag_type: global___BuildVersioningTagsType.ValueType
    """The type of the tags"""
    use_cloud_provider_storage: builtins.bool
    """When true, will use the cloud provider URL"""
    def __init__(
        self,
        *,
        model_id: builtins.str = ...,
        build_id: builtins.str = ...,
        tag: builtins.str = ...,
        environment_id: builtins.str = ...,
        tag_type: global___BuildVersioningTagsType.ValueType = ...,
        use_cloud_provider_storage: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["cloud_provider_option", b"cloud_provider_option", "use_cloud_provider_storage", b"use_cloud_provider_storage"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["build_id", b"build_id", "cloud_provider_option", b"cloud_provider_option", "environment_id", b"environment_id", "model_id", b"model_id", "tag", b"tag", "tag_type", b"tag_type", "use_cloud_provider_storage", b"use_cloud_provider_storage"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["cloud_provider_option", b"cloud_provider_option"]) -> typing_extensions.Literal["use_cloud_provider_storage"] | None: ...

global___BuildVersioningUrlParams = BuildVersioningUrlParams

class BuildVersioningTags(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_ID_FIELD_NUMBER: builtins.int
    BUILD_ID_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    TAGS_DEFINITION_FIELD_NUMBER: builtins.int
    ENVIRONMENT_ID_FIELD_NUMBER: builtins.int
    model_id: builtins.str
    """Model ID"""
    build_id: builtins.str
    """Build Id"""
    @property
    def tags(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Tags to save under"""
    @property
    def tags_definition(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BuildVersioningTagsDefinition]:
        """The tags definition"""
    environment_id: builtins.str
    """Environment to save the artifact under"""
    def __init__(
        self,
        *,
        model_id: builtins.str = ...,
        build_id: builtins.str = ...,
        tags: collections.abc.Iterable[builtins.str] | None = ...,
        tags_definition: collections.abc.Iterable[global___BuildVersioningTagsDefinition] | None = ...,
        environment_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["build_id", b"build_id", "environment_id", b"environment_id", "model_id", b"model_id", "tags", b"tags", "tags_definition", b"tags_definition"]) -> None: ...

global___BuildVersioningTags = BuildVersioningTags

class BuildVersioningTagsDefinition(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TAGS_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    @property
    def tags(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Tags to save under"""
    type: global___BuildVersioningTagsType.ValueType
    """The type of the tags"""
    def __init__(
        self,
        *,
        tags: collections.abc.Iterable[builtins.str] | None = ...,
        type: global___BuildVersioningTagsType.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["tags", b"tags", "type", b"type"]) -> None: ...

global___BuildVersioningTagsDefinition = BuildVersioningTagsDefinition

class BuildVersioningTagsProperties(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TAG_FIELD_NUMBER: builtins.int
    DOWNLOAD_URL_FIELD_NUMBER: builtins.int
    FILE_SIZE_FIELD_NUMBER: builtins.int
    TAG_TYPE_FIELD_NUMBER: builtins.int
    EXTERNAL_DOWNLOAD_URL_FIELD_NUMBER: builtins.int
    UI_DIRECT_LINK_FIELD_NUMBER: builtins.int
    tag: builtins.str
    """Tag to save under"""
    download_url: builtins.str
    """Download url - to download the wanted artifact"""
    file_size: builtins.int
    """The size of the file"""
    tag_type: global___BuildVersioningTagsType.ValueType
    """The type of the tag"""
    @property
    def external_download_url(self) -> global___ExternalUrl:
        """Download url - to download the wanted artifact - contains also the original host name if needed."""
    ui_direct_link: builtins.str
    """The url to the file"""
    def __init__(
        self,
        *,
        tag: builtins.str = ...,
        download_url: builtins.str = ...,
        file_size: builtins.int = ...,
        tag_type: global___BuildVersioningTagsType.ValueType = ...,
        external_download_url: global___ExternalUrl | None = ...,
        ui_direct_link: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["external_download_url", b"external_download_url"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["download_url", b"download_url", "external_download_url", b"external_download_url", "file_size", b"file_size", "tag", b"tag", "tag_type", b"tag_type", "ui_direct_link", b"ui_direct_link"]) -> None: ...

global___BuildVersioningTagsProperties = BuildVersioningTagsProperties

class ExternalUrl(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    URL_FIELD_NUMBER: builtins.int
    ORIGINAL_HOSTNAME_FIELD_NUMBER: builtins.int
    url: builtins.str
    """The url"""
    original_hostname: builtins.str
    """If exists the original_host_name"""
    def __init__(
        self,
        *,
        url: builtins.str = ...,
        original_hostname: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["original_hostname", b"original_hostname", "url", b"url"]) -> None: ...

global___ExternalUrl = ExternalUrl

class BuildArtifactPathSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUILD_ID_FIELD_NUMBER: builtins.int
    MODEL_ID_FIELD_NUMBER: builtins.int
    TAG_FIELD_NUMBER: builtins.int
    KEY_PATH_FIELD_NUMBER: builtins.int
    build_id: builtins.str
    """The build ID associated with the artifact"""
    model_id: builtins.str
    """The model ID associated with the artifact"""
    tag: builtins.str
    """The tag used to identify the artifact"""
    key_path: builtins.str
    """The path where the artifact is stored"""
    def __init__(
        self,
        *,
        build_id: builtins.str = ...,
        model_id: builtins.str = ...,
        tag: builtins.str = ...,
        key_path: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["build_id", b"build_id", "key_path", b"key_path", "model_id", b"model_id", "tag", b"tag"]) -> None: ...

global___BuildArtifactPathSpec = BuildArtifactPathSpec

class DataArtifact(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DataArtifact = DataArtifact

class CodeArtifact(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___CodeArtifact = CodeArtifact

class FileArtifact(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___FileArtifact = FileArtifact

class BuildArtifact(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    DATA_ARTIFACT_FIELD_NUMBER: builtins.int
    CODE_ARTIFACT_FIELD_NUMBER: builtins.int
    FILE_ARTIFACT_FIELD_NUMBER: builtins.int
    PATH_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The name used to identify the artifact"""
    @property
    def data_artifact(self) -> global___DataArtifact:
        """The artifact is a data artifact"""
    @property
    def code_artifact(self) -> global___CodeArtifact:
        """The artifact is a code artifact"""
    @property
    def file_artifact(self) -> global___FileArtifact:
        """The artifact is a file artifact"""
    path: builtins.str
    """The path where the artifact is stored"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        data_artifact: global___DataArtifact | None = ...,
        code_artifact: global___CodeArtifact | None = ...,
        file_artifact: global___FileArtifact | None = ...,
        path: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["code_artifact", b"code_artifact", "data_artifact", b"data_artifact", "file_artifact", b"file_artifact", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["code_artifact", b"code_artifact", "data_artifact", b"data_artifact", "file_artifact", b"file_artifact", "name", b"name", "path", b"path", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["data_artifact", "code_artifact", "file_artifact"] | None: ...

global___BuildArtifact = BuildArtifact
